import unittest
from tamilrulepy.euphonic import get 
from tamilrulepy.euphonic.stackwords import Maintainer



def words_data(words_list):
    return Maintainer(words_list,lang='ta')



class Test_Punarcci(unittest.TestCase):

    def test_punarcci_rule_1(self):
        
        # true case 
        
        sample1 = get( ["ஆ","இன்","ஐ"] ) 
        self.assertIn( ['ஆவினை'] , sample1)
        self.assertIn( ['ஆனை'] , sample1)

        sample2 = get( ["தா","இன்","ஐ"] ) 
        self.assertIn( ['தாவினை'], sample2)
        self.assertIn( ['தானை'] , sample2)

        sample3 = get([ 'விழா',"இன்",'சிறப்பு' ]) 
        self.assertIn( ['விழாவின்சிறப்பு'] , sample3)
        self.assertIn( ['விழான்சிறப்பு'] , sample3)
        
        sample4 = get( ["கலா","இன்","கலை"] ) 
        self.assertIn( ['கலாவின்கலை'] , sample4)
        self.assertIn( ['கலான்கலை'] , sample4)

        sample5 = get( ["நிலா","இன்","ஒளி"] ) 
        self.assertIn( ['நிலாவினொளி'] , sample5)
        self.assertIn( ['நிலானொளி'] , sample5)

        sample6 = get([ 'பலா',"இன்",'கனி' ]) 
        self.assertIn( ['பலாவின்கனி'] , sample6)
        self.assertIn( ['பலான்கனி'] , sample6)
        
        sample7 = get( ["மா","இன்","தலை"] ) 
        self.assertIn( ['மாவின்தலை'] , sample7)
        self.assertIn( ['மான்தலை'] , sample7)

        sample8 = get([ 'ஆ',"இன்",'பால்' ]) 
        self.assertIn( ['ஆவின்பால்'], sample8)
        self.assertIn( ['ஆன்பால்'] , sample8)



    def test_punarcci_rule_2(self):
        
        sample1 = get(  ["பத்து","இன்","உழக்கு"] ) 
        self.assertIn(['பதிற்றுழக்கு'],sample1) 

        sample2 = get(  ["பத்து","இன்","அகழ்"] ) 
        self.assertIn(['பதிற்றகழ்'],sample2) 

        sample3 = get( [ 'பத்து',"இன்",'ஆலாக்கு' ]) 
        self.assertIn(['பதிற்றாலாக்கு'],sample3) 

        sample4 = get(  ["பத்து","இன்","அங்குலம்"] ) 
        self.assertIn(['பதிற்றங்குலம்'],sample4) 

        # type 2

        sample11 = get(  ["பத்து","இன்","ஏழு"] ) 
        self.assertIn(['பதிற்றேழு'],sample11) 

        sample12 = get(  ["பத்து","இன்","ஒன்று"] ) 
        self.assertIn(['பதிற்றொன்று'],sample12) 

        sample13 = get( [ 'பத்து',"இன்",'ஒன்பது' ]) 
        self.assertIn(['பதிற்றொன்பது'],sample13) 





    def test_punarcci_rule_3(self):
        sample0 = get(["அவை","வற்று","ஐ"])
        self.assertIn(['அவையற்றை'],sample0) 
        
        sample1 = get(  ["அவை","வற்று","கு"] ) 
        self.assertIn(['அவையற்றுக்கு'],sample1) 

        sample2 = get(  ["இவை","வற்று","ஐ"] ) 
        self.assertIn(['இவையற்றை'],sample2) 

        sample3 = get( [ 'அவை',"வற்று",'ஆல்' ]) 
        self.assertIn(['அவையற்றால்'],sample3) 
        
        sample4 = get(["அவை","வற்று","இன்"])
        self.assertIn(['அவையற்றின்'],sample4) 
        
        # TODO
        #sample2 = get(  ["இவை","வற்று","கண்"] ) 
        #self.assertIn(['அவையற்றின்'],sample2) 

        #sample3 = get( [ 'அவை',"வற்று",'கண்' ]) 
        #self.assertIn(['பதிற்றாலாக்கு'],sample3) 

        sample10 = get(  ["இவை","வற்று","இன்"] ) 
        self.assertIn(['இவையற்றின்'],sample10) 

        sample11 = get(  ["உ","வற்று","ஐ"] ) 
        self.assertIn(['உவற்றை'],sample11) 

        sample12 = get(  ["இ","வற்று","ஐ"] ) 
        self.assertIn(['இவற்றை'],sample12) 

        sample13 = get( [ 'அ',"வற்று",'ஐ' ]) 
        self.assertIn(['அவற்றை'],sample13) 
        
        
    def test_punarcci_rule_4(self):
        
        sample0 = get([ "விள","இன்","கு" ])
        self.assertIn(['விளவிற்கு'],sample0) 
        
        sample1 = get(["அவன்","இன்","கு"]) 
        self.assertIn(['அவனிற்கு'],sample1) 

        sample2 = get(["அவள்","இன்","கு"]) 
        self.assertIn(['அவளிற்கு'],sample2) 

        sample3 = get([ 'மகம்',"இன்",'கு' ]) 
        self.assertIn(['மகவிற்கு'],sample3) 
        
        sample4 = get(["கதவு","இன்","கு"])
        self.assertIn(['கதவிற்கு'],sample4) 
        



    def test_punarcci_rule_5(self):
        
        sample0 = get(['பரணி','ஆன்','கொண்டான்'])
        self.assertIn(['பரணியாற்கொண்டான்'],sample0) 
        
        sample1 = get(['தடத்தின்','ஆன்','கொண்டான்']) 
        self.assertIn(['தடத்தினாற்கொண்டான்'],sample1) 

        sample2 = get(['தரனி','ஆன்','கொண்டான்']) 
        self.assertIn(['தரனியாற்கொண்டான்'],sample2) 

        sample3 = get(['ஊரனி','ஆன்','கொண்டான்']) 
        self.assertIn(['ஊரனியாற்கொண்டான்'],sample3) 
        
        sample4 = get(['அதன்','ஆன்','கொண்டான்'])
        self.assertIn(['அதனாற்கொண்டான்'],sample4) 
        

    def test_punarcci_rule_6(self):
    
        #TODO
        result = get(["மரம்","அத்து","ஐ"])
        #self.assertIn(['மரத்தை'],result) 

        sample1 = get( [ 'மடம்','அத்து','கை' ]  ) 
        self.assertIn( ['மடத்துக்கை'] ,sample1) 

        sample2 = get(['மரம்','அத்து','கேணி' ] ) 
        self.assertIn(['மரத்துக்கேணி'],sample2) 

        sample3 = get( ['மரம்','அத்து','கை'] ) 
        self.assertIn(['மரத்துக்கை'],sample3) 
        
        sample4 = get( ['மரம்','அத்து','கிளை'] )
        self.assertIn(['மரத்துக்கிளை'],sample4) 
   


    def test_punarcci_rule_7(self):

        sample0 = get([ 'ஆடி','இக்கு','கொண்டான்'])
        self.assertIn(['ஆடிக்குக்கொண்டான்'],sample0) 
        
        sample1 = get([ 'சித்திரை','இக்கு','கொண்டான்']) 
        self.assertIn( ['சித்திரைக்குக்கொண்டான்'] ,sample1) 

        sample2 = get(['ஆடி','இக்கு','கொண்டான்' ] ) 
        self.assertIn(['ஆடிக்குக்கொண்டான்'],sample2) 

        sample3 = get( ['ஆடி','இக்கு','தந்தான்'] ) 
        self.assertIn(['ஆடிக்குதந்தான்'],sample3) 
        
        sample4 = get( ['ஆடி','இக்கு','போயினான்'] )
        self.assertIn(['ஆடிக்குப்போயினான்'],sample4) 
           
        sample5 = get([ 'சித்திரை','இக்கு','சன்றான்']) 
        self.assertIn( ['சித்திரைக்குச்சன்றான்'] ,sample5) 



    def test_punarcci_rule_8(self):
        result = get([ 'சித்திரை','இக்கு','கொண்டான்'])
        self.assertIn(['சித்திரைக்குக்கொண்டான்'],result)   


    def test_punarcci_rule_9(self):
        result = get([ 'குன்று','அக்கு','குடி'  ])
        self.assertIn(['குன்றக்குடி'],result) 

    def test_punarcci_rule_10(self):                    
        result = get([ 'புளி','அம்','கிளை' ])
        self.assertIn(['புளியங்கிளை'],result) 
            
    
    def test_punarcci_rule_11(self):
       
        sample0 = get(['புளி','அம்','நுனி'])
        self.assertIn(['புளியநுனி'],sample0) 
        
        sample1 = get( ['புளி','அம்','தலை']  ) 
        self.assertIn( ['புளியந்தலை'] ,sample1) 

        sample2 = get(['புளி','அம்','கோது' ] ) 
        self.assertIn(['புளியங்கோது'],sample2) 

        sample3 = get( ['புளி','அம்','செதில்'] ) 
        self.assertIn(['புளியஞ்செதில்'],sample3) 
        
        sample4 = get( ['புளி','அம்','தோல்'] )
        self.assertIn(['புளியந்தோல்'],sample4) 
        
        sample5 = get( ['புளி','அம்','பட்டை'] )
        self.assertIn(['புளியப்பட்டை'],sample5) 
   


    def test_punarcci_rule_12(self):
        
        sample0 = get([ 'வீடு','இன்','தென்புறம்' ])
        self.assertIn(['வீட்டின்தென்புறம்'],sample0) 
        
        sample1 = get( [ 'மடம்','அத்து','கை' ]  ) 
        self.assertIn( ['மடத்துக்கை'] ,sample1) 

        sample2 = get(['மரம்','அத்து','கேணி' ] ) 
        self.assertIn(['மரத்துக்கேணி'],sample2) 

        sample3 = get( ['மரம்','அத்து','கை'] ) 
        self.assertIn(['மரத்துக்கை'],sample3) 
        
        sample4 = get( ['மரம்','அத்து','கிளை'] )
        self.assertIn(['மரத்துக்கிளை'],sample4) 
   

    def test_punarcci_rule_13(self):
      
        sample0 = get( [ 'மடம்','அத்து','கேணி' ] )
        self.assertIn(['மடத்துக்கேணி'],sample0) 
        
        sample1 = get( [ 'மடம்','அத்து','கை' ]  ) 
        self.assertIn( ['மடத்துக்கை'] ,sample1) 

        sample2 = get(['மரம்','அத்து','கேணி' ] ) 
        self.assertIn(['மரத்துக்கேணி'],sample2) 

        sample3 = get( ['மரம்','அத்து','கை'] ) 
        self.assertIn(['மரத்துக்கை'],sample3) 
        
        sample4 = get( ['மரம்','அத்து','கிளை'] )
        self.assertIn(['மரத்துக்கிளை'],sample4) 

    def test_punarcci_rule_14_to_16(self):
        result = get([ 'பு' ])
        self.assertIn(['புகாரம்'],result) 
        self.assertIn(['புகரம்'],result) 
        self.assertIn(['புஃகான்'],result) 
        
        result = get([ 'சு' ])
        self.assertIn(['சுகாரம்'],result) 
        self.assertIn(['சுகரம்'],result) 
        self.assertIn(['சுஃகான்'],result) 

        result = get([ 'ஐ' ])
        self.assertIn(['ஐகாரம்'],result) 
        self.assertIn(['ஐகான்'],result) 

        result = get([ 'ஔ' ])
        self.assertIn(['ஔகாரம்'],result) 
        self.assertIn(['ஔகான்'],result) 

    def test_punarcci_rule_17(self):
        result = get(['கடல்','அலை'])
        self.assertIn(['கடலலை'],result) 
    
    def test_punarcci_rule_18(self):
        result = get([ 'ஆவ்','இலை' ])
        self.assertIn(['ஆவிலை'],result) 




import unittest
from tamilrulepy.euphonic.previous import remove_ending_consonent_m 
from tamilrulepy.euphonic.stackwords import Maintainer


def words_data(words_list):
    return Maintainer(words_list,lang='ta')



class Test_Euphonic_commin(unittest.TestCase):


    def test_remove_ending_consonent_m(self):
        # true case 
        case1 = remove_ending_consonent_m( words_data(["மரம்","அத்து","ஐ"]) )
        self.assertTrue(case1[0])

        case2 = remove_ending_consonent_m( words_data(["மகம்","அத்து","கிளை"]) )
        self.assertTrue(case2[0])

        case3 = remove_ending_consonent_m( words_data([ 'மடம்','அத்து','கேணி' ]) )
        self.assertTrue(case3[0])
        
        case4 = remove_ending_consonent_m( words_data([ 'பவளம்','இதல்', ]) )
        self.assertTrue(case4[0])
  
        case4 = remove_ending_consonent_m( words_data([ 'குளம்','கரை', ]) )
        self.assertTrue(case4[0])
  

        # false case

        case10 = remove_ending_consonent_m( words_data([ 'குன்று','அக்கு','குடி' ]) )
        self.assertFalse(case10[0])
  
        case11 = remove_ending_consonent_m( words_data([ 'மா','மரம்' ]) )
        self.assertFalse(case11[0])
  





import tamil

# பைத்தானில் தமிழ் மொழியை உள்ளீடு செய்வதற்கு இது பயன்படுத்தப்பெற்றுள்ளது.

sentence = "வலை, வானம், விலை, வீடு, வெள்ளி, வேம்பு, வையம், வௌவுதல், வேவ்வாறு ஏ ஐ ஓ யான், யாண்டு, யாறு, கொல்லாது"
# இது தரவை உள்ளீடாய்த் தருவதற்குரிய பகுதி்.


def uyirezhuthu_check(word):
    """
    Checks if the first letter of a given Tamil word is a 'உயிரெழுத்து(Uyirezhuthu)' (vowel) character.

    'உயிரெழுத்து' refers to the vowels in the Tamil script. This function checks whether
    the first letter of the input word is one of the following Tamil vowels:
    அ, ஆ, இ, ஈ, உ, ஊ, ஏ, ஐ, ஒ, ஓ, or ஔ.

    Parameters:
    word (str): The input Tamil word to be checked.

    Returns:
    bool: True if the first letter of the word is a Tamil vowel ('உயிரெழுத்து Uyirezhuthu').
    None: If the first letter is not a Tamil vowel.
    """
    உயிர்_முதலெழுத்து = ["அ", "ஆ", "இ", "ஈ", "உ", "ஊ", "ஏ", "ஐ", "ஒ", "ஓ", "ஔ"]
    first_letter = tamil.utf8.get_letters(word)[0]
    if first_letter in உயிர்_முதலெழுத்து:
        return True
    else:
        return None


# இது உயிர் எழுத்துக்களை முதலாகக் கொண்ட நிரலாக்கம்.


def uyirmei_ka_check(word):
    """
    Checks if the first letter of a given Tamil word is a 'உயிர்மெய் (Uyirmei)' (consonant-vowel)
    character from the 'க் (Ka)' series in the Tamil script.

    This function verifies whether the first letter of the input word is one of the
    following consonant-vowel combinations starting with 'க் (Ka)' in the Tamil script:
    க, கா, கி, கீ, கு, கூ, கெ, கே, கை, கொ, கோ, or கௌ.

    Parameters:
    word (str): The input Tamil word to be checked.

    Returns:
    bool: True if the first letter of the word is one of the specified 'க் (Ka)' series characters.
    None: If the first letter is not a 'க் (Ka)' series consonant-vowel combination.
    """
    உயிர்மெய்_முதலெழுத்து_கவரிசை = [
        "க",
        "கா",
        "கி",
        "கீ",
        "கு",
        "கூ",
        "கெ",
        "கே",
        "கை",
        "கொ",
        "கோ",
        "கௌ",
    ]
    first_letter = tamil.utf8.get_letters(word)[0]
    if first_letter in உயிர்மெய்_முதலெழுத்து_கவரிசை:
        return True
    else:
        return None


# இது கவரிசை எழுத்துக்களை முதலாகக் கொண்ட நிரலாக்கம்.


def uyirmei_sa_check(word):
    """
    Checks if the first letter of a given Tamil word is a 'உயிர்மெய் (Uyirmei)' (consonant-vowel)
    character from the 'ச் (Sa)' series in the Tamil script.

    This function verifies whether the first letter of the input word is one of the
    following consonant-vowel combinations starting with 'ச் (Sa)' in the Tamil script:
    ச, சா, சி, சீ, சு, சூ, செ, சே, சொ, சோ.

    Parameters:
    word (str): The input Tamil word to be checked.

    Returns:
    bool: True if the first letter of the word is one of the specified 'ச் (Sa)' series characters.
    None: If the first letter is not a 'ச் (Sa)' series consonant-vowel combination.
    """
    உயிர்மெய்_முதலெழுத்து_சவரிசை = [
        "ச",
        "சா",
        "சி",
        "சீ",
        "சு",
        "சூ",
        "செ",
        "சே",
        "சொ",
        "சோ",
    ]
    first_letter = tamil.utf8.get_letters(word)[0]
    if first_letter in உயிர்மெய்_முதலெழுத்து_சவரிசை:
        return True
    else:
        return None


# இது சவரிசை எழுத்துக்களை முதலாகக் கொண்ட நிரலாக்கம்.


def uyirmei_nga_check(word):
    """
    Checks if the first letter of a given Tamil word is a 'உயிர்மெய் (Uyirmei)' (consonant-vowel)
    character from the 'ஞ் (Nga)' series in the Tamil script.

    This function verifies whether the first letter of the input word is one of the
    following consonant-vowel combinations starting with 'ஞ் (Nga)' in the Tamil script:
    ஞா, ஞெ, ஞொ.

    Parameters:
    word (str): The input Tamil word to be checked.

    Returns:
    bool: True if the first letter of the word is one of the specified 'ஞ் (Nga)' series characters.
    None: If the first letter is not a 'ஞ் (Nga)' series consonant-vowel combination.
    """
    உயிர்மெய்_முதலெழுத்து_ஞவரிசை = ["ஞா", "ஞெ", "ஞொ"]
    first_letter = tamil.utf8.get_letters(word)[0]
    if first_letter in உயிர்மெய்_முதலெழுத்து_ஞவரிசை:
        return True
    else:
        return None


# இது ஞவரிசை எழுத்துக்களை முதலாகக் கொண்ட நிரலாக்கம்.


def uyirmei_ta_check(word):
    """
    Checks if the first letter of a given Tamil word is a 'உயிர்மெய் (Uyirmei)' (consonant-vowel)
    character from the 'த் (Tha)' series in the Tamil script.

    This function verifies whether the first letter of the input word is one of the
    following consonant-vowel combinations starting with 'த் (Tha)' in the Tamil script:
    த, தா, தி, தீ, து, தூ, தெ, தே, தை, தொ, தோ, தௌ.

    Parameters:
    word (str): The input Tamil word to be checked.

    Returns:
    bool: True if the first letter of the word is one of the specified 'த் (Tha)' series characters.
    None: If the first letter is not a 'த் (Tha)' series consonant-vowel combination.
    """
    உயிர்மெய்_முதலெழுத்து_தவரிசை = [
        "த",
        "தா",
        "தி",
        "தீ",
        "து",
        "தூ",
        "தெ",
        "தே",
        "தை",
        "தொ",
        "தோ",
        "தௌ",
    ]
    first_letter = tamil.utf8.get_letters(word)[0]
    if first_letter in உயிர்மெய்_முதலெழுத்து_தவரிசை:
        return True
    else:
        return None


# இது தவரிசை எழுத்துக்களை முதலாகக் கொண்ட நிரலாக்கம்.


def uyirmei_na_check(word):
    """
    Checks if the first letter of a given Tamil word is a 'உயிர்மெய் (Uyirmei)' (consonant-vowel)
    character from the 'ந் (Na)' series in the Tamil script.

    This function verifies whether the first letter of the input word is one of the
    following consonant-vowel combinations starting with 'ந் (Na)' in the Tamil script:
    ந, நா, நி, நீ, நு, நூ, நெ, நே, நை, நொ, நோ, நௌ.

    Parameters:
    word (str): The input Tamil word to be checked.

    Returns:
    bool: True if the first letter of the word is one of the specified 'ந் (Na)' series characters.
    None: If the first letter is not a 'ந் (Na)' series consonant-vowel combination.
    """
    உயிர்மெய்_முதலெழுத்து_நவரிசை = [
        "ந",
        "நா",
        "நி",
        "நீ",
        "நு",
        "நூ",
        "நெ",
        "நே",
        "நை",
        "நொ",
        "நோ",
        "நௌ",
    ]
    first_letter = tamil.utf8.get_letters(word)[0]
    if first_letter in உயிர்மெய்_முதலெழுத்து_நவரிசை:
        return True
    else:
        return None


# இது நவரிசை எழுத்துக்களை முதலாகக் கொண்ட நிரலாக்கம்.


def uyirmei_pa_check(word):
    """
    Checks if the first letter of a given Tamil word is a 'உயிர்மெய் (Uyirmei)' (consonant-vowel)
    character from the 'ப் (Pa)' series in the Tamil script.

    This function verifies whether the first letter of the input word is one of the
    following consonant-vowel combinations starting with 'ப் (Pa)' in the Tamil script:
    ப, பா, பி, பீ, பு, பூ, பெ, பே, பை, பொ, போ, பௌ.

    Parameters:
    word (str): The input Tamil word to be checked.

    Returns:
    bool: True if the first letter of the word is one of the specified 'ப் (Pa)' series characters.
    None: If the first letter is not a 'ப் (Pa)' series consonant-vowel combination.
    """
    உயிர்மெய்_முதலெழுத்து_பவரிசை = [
        "ப",
        "பா",
        "பி",
        "பீ",
        "பு",
        "பூ",
        "பெ",
        "பே",
        "பை",
        "பொ",
        "போ",
        "பௌ",
    ]
    first_letter = tamil.utf8.get_letters(word)[0]
    if first_letter in உயிர்மெய்_முதலெழுத்து_பவரிசை:
        return True
    else:
        return None


# இது பவரிசை எழுத்துக்களை முதலாகக் கொண்ட நிரலாக்கம்.


def uyirmei_ma_check(word):
    """
    Checks if the first letter of a given Tamil word is a 'உயிர்மெய் (Uyirmei)' (consonant-vowel)
    character from the 'ம் (Ma)' series in the Tamil script.

    This function verifies whether the first letter of the input word is one of the
    following consonant-vowel combinations starting with 'ம் (Ma)' in the Tamil script:
    ம, மா, மி, மீ, மு, மூ, மெ, மே, மை, மொ, மோ, மௌ.

    Parameters:
    word (str): The input Tamil word to be checked.

    Returns:
    bool: True if the first letter of the word is one of the specified 'ம் (Ma)' series characters.
    None: If the first letter is not a 'ம் (Ma)' series consonant-vowel combination.
    """
    உயிர்மெய்_முதலெழுத்து_மவரிசை = [
        "ம",
        "மா",
        "மி",
        "மீ",
        "மு",
        "மூ",
        "மெ",
        "மே",
        "மை",
        "மொ",
        "மோ",
        "மௌ",
    ]
    first_letter = tamil.utf8.get_letters(word)[0]
    if first_letter in உயிர்மெய்_முதலெழுத்து_மவரிசை:
        return True
    else:
        return None


# இது மவரிசை எழுத்துக்களை முதலாகக் கொண்ட நிரலாக்கம்.


def uyirmei_ya_check(word):
    """
    Checks if the first letter of a given Tamil word is a 'உயிர்மெய் (Uyirmei)' (consonant-vowel)
    character from the 'ய் (Ya)' series in the Tamil script.

    This function verifies whether the first letter of the input word is the consonant-vowel
    combination starting with 'ய் (Ya)' in the Tamil script: யா.

    Parameters:
    word (str): The input Tamil word to be checked.

    Returns:
    bool: True if the first letter of the word is the specified 'ய் (Ya)' series character.
    None: If the first letter is not the 'ய் (Ya)' series consonant-vowel combination.
    """
    உயிர்மெய்_முதலெழுத்து_யவரிசை = ["யா"]
    first_letter = tamil.utf8.get_letters(word)[0]
    if first_letter in உயிர்மெய்_முதலெழுத்து_யவரிசை:
        return True
    else:
        return None


# இது யவரிசை எழுத்துக்களை முதலாகக் கொண்ட நிரலாக்கம்.


def uyirmei_va_check(word):
    """
    Checks if the first letter of a given Tamil word is a 'உயிர்மெய் (Uyirmei)' (consonant-vowel)
    character from the 'வ் (Va)' series in the Tamil script.

    This function verifies whether the first letter of the input word is one of the
    following consonant-vowel combinations starting with 'வ் (Va)' in the Tamil script:
    வ, வா, வி, வீ, வெ, வே, வை, வௌ.

    Parameters:
    word (str): The input Tamil word to be checked.

    Returns:
    bool: True if the first letter of the word is one of the specified 'வ் (Va)' series characters.
    None: If the first letter is not a 'வ் (Va)' series consonant-vowel combination.
    """
    உயிர்மெய்_முதலெழுத்து_வவரிசை = ["வ", "வா", "வி", "வீ", "வெ", "வே", "வை", "வௌ"]
    first_letter = tamil.utf8.get_letters(word)[0]
    if first_letter in உயிர்மெய்_முதலெழுத்து_வவரிசை:
        return True
    else:
        return None


# இது வவரிசை எழுத்துக்களை முதலாகக் கொண்ட நிரலாக்கம்.


# இது வவரிசை எழுத்துக்களை முதலாகக் கொண்ட நிரலாக்கம்.

# இது உயிர் எழுத்தில் முடியும் உயிர்மெய் எழுத்துக்களைப் பொருத்திப் பார்ப்பதற்குரிய நிரலாகும்.
# வேவ்வாறு True > False
# இதுகுறித்த தெளிவு தேவை. இதற்குப் புதியதாக விதி எழுதுதல் வேண்டும்.



import tamil

# 1) க் + க
# ஒரு சொல்லில் க் எழுத்துக்குப் பின்பு க எழுத்து மயங்கி வரும். எ-டு. அக்கா, ஆக்கம்

# word=input("Enter the word to check: ")


def meymayakkam_checker(
    word_letters,
    letter,
    allowed_list,
    Mei=[
        "க்",
        "ங்",
        "ச்",
        "ஞ்",
        "ட்",
        "ண்",
        "த்",
        "ந்",
        "ப்",
        "ம்",
        "ய்",
        "ர்",
        "ல்",
        "வ்",
        "ழ்",
        "ள்",
        "ற்",
        "ன்",
    ],
):
    """
    meymayakkam_checker is generic utility function used to do the regular check.
    All the meymayakkam rule follows a similar pattern like once a mei words came it needs to be followed with some words

    This function helps to do the check that and since it is writen in generic way, all the rules will be using this for checking.

        Parameters:
            word_letters list(str) : word as a list with individual letters as elements
            letter (str) : Rule specific mei letter we want to check
            allowed_list list(str) : list of letters which are allowed to follow the mei letter as per tholkappiyar rule

        Returns :
            result (bool):
                True - If the mei letter is followed by correct allowed letters only
                False - If the mei letter is not followed by the allowed letters
    """
    ind = word_letters.index(letter)
    if (
        word_letters[ind + 1] in Mei
    ):  # Need to check on this point since some are overlapping. we can use sets here
        return False
    else:
        root_words = tamil.utf8.splitMeiUyir(word_letters[ind + 1])
        if type(root_words) == tuple:
            root_last = root_words[0]
        else:
            root_last = root_words
        if root_last in allowed_list:
            return True
        else:
            return False


# "மெய்ம்மயக்கம்1": "க்+க"
def meymayakkam1(word):
    """
    meymayakkam1 is the implementation of Tholkappiyar rule of mei mayakkam for the letter "க்"

    The Letter  "க்" must be followed by any derivatives of "க" i.e க,கா,கி,...

        Parameters:
            word (str) : word you want to check the grammar correctness

        Returns :
            result (bool):
                True - The rule matches and it it grammatically correct for "க்"
                False - The rule matches and it is not correct for the letter "க்"
                None - The rule doesn't apply since there is no "க்" in the word
    """
    letters = tamil.utf8.get_letters(word)
    if "க்" in letters and letters.index("க்") != len(letters) - 1:
        return meymayakkam_checker(letters, "க்", ["க்"])
    else:
        return None


# "மெய்ம்மயக்கம்2": "ங்+கங"
def meymayakkam2(word):
    """
    meymayakkam2 is the implementation of Tholkappiyar rule of mei mayakkam for the letter "ங்"

    The Letter  "ங்" must be followed by any derivatives of "க", "ங" i.e க,கா,கி,...  (or)ங, ஙா ஙி ,...

        Parameters:
            word (str) : word you want to check the grammar correctness

        Returns :
            result (bool):
                True - The rule matches and it it grammatically correct for "ங்"
                False - The rule matches and it is not correct for the letter "ங்"
                None - The rule doesn't apply since there is no "ங்" in the word
    """
    letters = tamil.utf8.get_letters(word)
    if "ங்" in letters and letters.index("ங்") != len(letters) - 1:
        return meymayakkam_checker(letters, "ங்", ["க்", "ங்"])
    else:
        return None


# "மெய்ம்மயக்கம்3": "ச்+ச"
def meymayakkam3(word):
    """
    meymayakkam3 is the implementation of Tholkappiyar rule of mei mayakkam for the letter "ச்"

    The Letter  "ச்" must be followed by any derivatives of "ச" i.e ச, சா, சி, சீ,...

        Parameters:
            word (str) : word you want to check the grammar correctness

        Returns :
            result (bool):
                True - The rule matches and it it grammatically correct for "ச்"
                False - The rule matches and it is not correct for the letter "ச்"
                None - The rule doesn't apply since there is no "ச்" in the word
    """
    letters = tamil.utf8.get_letters(word)
    if "ச்" in letters and letters.index("ச்") != len(letters) - 1:
        return meymayakkam_checker(letters, "ச்", ["ச்"])
    else:
        return None


# "மெய்ம்மயக்கம்4": "ஞ்+சஞய
def meymayakkam4(word):
    """
    meymayakkam4 is the implementation of Tholkappiyar rule of mei mayakkam for the letter "ஞ்"

    The Letter  "ஞ்" must be followed by any derivatives of "ச","ஞ","ய" (i.e) ச, சா, சி,... (or) ஞ 	ஞா 	ஞி,... (or) ய 	யா 	யி,...

        Parameters:
            word (str) : word you want to check the grammar correctness

        Returns :
            result (bool):
                True - The rule matches and it it grammatically correct for "ஞ்"
                False - The rule matches and it is not correct for the letter "ஞ்"
                None - The rule doesn't apply since there is no "ஞ்"  in the word
    """
    letters = tamil.utf8.get_letters(word)
    if "ஞ்" in letters and letters.index("ஞ்") != len(letters) - 1:
        return meymayakkam_checker(letters, "ஞ்", ["ச்", "ஞ்", "ய்"])
    else:
        return None


# "மெய்ம்மயக்கம்5": "ட்+கசடப"
def meymayakkam5(word):
    """
    meymayakkam5 is the implementation of Tholkappiyar rule of mei mayakkam for the letter "ட்"

    The Letter  "ட்" must be followed by any derivatives of "க", "ச", "ட", "ப" (i.e) க 	கா 	கி,... (or) ... (or) ய 	யா 	யி,...

        Parameters:
            word (str) : word you want to check the grammar correctness

        Returns :
            result (bool):
                True - The rule matches and it it grammatically correct for "ட்"
                False - The rule matches and it is not correct for the letter "ட்"
                None - The rule doesn't apply since there is no "ட்"  in the word
    """
    letters = tamil.utf8.get_letters(word)
    if "ட்" in letters and letters.index("ட்") != len(letters) - 1:
        return meymayakkam_checker(letters, "ட்", ["க்", "ச்", "ட்", "ப்"])
    else:
        return None


# "மெய்ம்மயக்கம்6": "ண்+கசஞடணபமயவ"
def meymayakkam6(word):
    """
    meymayakkam6 is the implementation of Tholkappiyar rule of mei mayakkam for the letter "ண்"

    The Letter  "ண்" must be followed by any derivatives of "க்","ச்","ஞ்","ட்","ண்","ப்","ம்","ய்","வ்" (i.e) க, கா,	கி... (or) ச,சா ,சி... (or) ஞ ,ஞா,ஞி... (or) ட ,டா,	டி... (or) ண ,ணா ,ணி... (or) ப,	பா,	பி... (or) ம,	மா,	மி ... (or) ய,	யா,	யி... (or) வ,	வா,	வி...

        Parameters:
            word (str) : word you want to check the grammar correctness

        Returns :
            result (bool):
                True - The rule matches and it it grammatically correct for "ண்"
                False - The rule matches and it is not correct for the letter "ண்"
                None - The rule doesn't apply since there is no "ண்" in the word

    """
    letters = tamil.utf8.get_letters(word)
    if "ண்" in letters and letters.index("ண்") != len(letters) - 1:
        return meymayakkam_checker(
            letters, "ண்", ["க்", "ச்", "ஞ்", "ட்", "ண்", "ப்", "ம்", "ய்", "வ்"]
        )
    else:
        return None


# "மெய்ம்மயக்கம்7": "த்+த"
def meymayakkam7(word):
    """
    meymayakkam7 is the implementation of Tholkappiyar rule of mei mayakkam for the letter "த்"

    The Letter  "த்" must be followed by any derivatives of "த்" (i.e) த,	த,,	தி...

        Parameters:
            word (str) : word you want to check the grammar correctness

        Returns :
            result (bool):
                True - The rule matches and it it grammatically correct for "த்"
                False - The rule matches and it is not correct for the letter "த்"
                None - The rule doesn't apply since there is no "த்" in the word
    """
    letters = tamil.utf8.get_letters(word)
    if "த்" in letters and letters.index("த்") != len(letters) - 1:
        return meymayakkam_checker(letters, "த்", ["த்"])
    else:
        return None


# "மெய்ம்மயக்கம்8": "ந்+தநய"
def meymayakkam8(word):
    """
    meymayakkam8 is the implementation of Tholkappiyar rule of mei mayakkam for the letter "ந்"

    The Letter  "ந்" must be followed by any derivatives of "த்","ந்","ய்" (i.e) த,	த,,	தி... (or) ந,	நா,	நி... (or) ய,	யா,	யி...

        Parameters:
            word (str) : word you want to check the grammar correctness

        Returns :
            result (bool):
                True - The rule matches and it it grammatically correct for "ந்"
                False - The rule matches and it is not correct for the letter "ந்"
                None - The rule doesn't apply since there is no "ந்" in the word
    """
    letters = tamil.utf8.get_letters(word)
    if "ந்" in letters and letters.index("ந்") != len(letters) - 1:
        return meymayakkam_checker(letters, "ந்", ["த்", "ந்", "ய்"])
    else:
        return None


# "மெய்ம்மயக்கம்9": "ப்+ப"
def meymayakkam9(word):
    """
    meymayakkam9 is the implementation of Tholkappiyar rule of mei mayakkam for the letter "ப்"

    The Letter  "ப்" must be followed by any derivatives of "ப்" (i.e) ப,	பா,	பி...

        Parameters:
            word (str) : word you want to check the grammar correctness

        Returns :
            result (bool):
                True - The rule matches and it it grammatically correct for "ப்"
                False - The rule matches and it is not correct for the letter "ப்"
                None - The rule doesn't apply since there is no "ப்" in the word
    """
    letters = tamil.utf8.get_letters(word)
    if "ப்" in letters and letters.index("ப்") != len(letters) - 1:
        return meymayakkam_checker(letters, "ப்", ["ப்"])
    else:
        return None


# "மெய்ம்மயக்கம்10": "ம்+பமயவ"
def meymayakkam10(word):
    """
    meymayakkam10 is the implementation of Tholkappiyar rule of mei mayakkam for the letter "ம்"

    The Letter  "ம்" must be followed by any derivatives of "ப்","ம்","ய்","வ்" (i.e) ப,	பா,	பி... (or) ம,	மா,	மி ... (or) ய,	யா,	யி... (or) வ,	வா,	வி...

        Parameters:
            word (str) : word you want to check the grammar correctness

        Returns :
            result (bool):
                True - The rule matches and it it grammatically correct for "ம்"
                False - The rule matches and it is not correct for the letter "ம்"
                None - The rule doesn't apply since there is no "ம்" in the word
    """
    letters = tamil.utf8.get_letters(word)
    if "ம்" in letters and letters.index("ம்") != len(letters) - 1:
        return meymayakkam_checker(letters, "ம்", ["ப்", "ம்", "ய்", "வ்"])
    else:
        return None


# "மெய்ம்மயக்கம்11": "ய்+கசதபஞநமயவங"
def meymayakkam11(word):
    """
    meymayakkam11 is the implementation of Tholkappiyar rule of mei mayakkam for the letter "ய்"

    The Letter  "ய்" must be followed by any derivatives of "க்","ங்","ச்","ஞ்","த்","ந்","ப்","ம்","ய்","வ்" (i.e) க, கா,	கி... (or) ங,ஙா,ஙி,... (or) ச,சா ,சி... (or) ஞ ,ஞா,ஞி... (or) த,	த,,	தி... (or) ந,	நா,	நி... (or) ப,	பா,	பி... (or) ம,	மா,	மி ... (or) ய,	யா,	யி... (or) வ,	வா,	வி...

        Parameters:
            word (str) : word you want to check the grammar correctness

        Returns :
            result (bool):
                True - The rule matches and it it grammatically correct for "ய்"
                False - The rule matches and it is not correct for the letter "ய்"
                None - The rule doesn't apply since there is no "ய்" in the word
    """
    letters = tamil.utf8.get_letters(word)
    if "ய்" in letters and letters.index("ய்") != len(letters) - 1:
        return meymayakkam_checker(
            letters, "ய்", ["க்", "ங்", "ச்", "ஞ்", "த்", "ந்", "ப்", "ம்", "ய்", "வ்"]
        )
    else:
        return None


# "மெய்ம்மயக்கம்12": "ர்+கசதபஞநமயவங"
def meymayakkam12(word):
    """
    meymayakkam12 is the implementation of Tholkappiyar rule of mei mayakkam for the letter "ர்"

    The Letter  "ர்" must be followed by any derivatives of "க்","ங்","ச்","ஞ்","த்","ந்","ப்","ம்","ய்","வ்" (i.e) க, கா,	கி... (or) ங,ஙா,ஙி,... (or) ச,சா ,சி... (or) ஞ ,ஞா,ஞி... (or) த,	த,,	தி... (or) ந,	நா,	நி... (or) ப,	பா,	பி... (or) ம,	மா,	மி ... (or) ய,	யா,	யி... (or) வ,	வா,	வி...

        Parameters:
            word (str) : word you want to check the grammar correctness

        Returns :
            result (bool):
                True - The rule matches and it it grammatically correct for "ர்"
                False - The rule matches and it is not correct for the letter "ர்"
                None - The rule doesn't apply since there is no "ர்" in the word
    """
    letters = tamil.utf8.get_letters(word)
    if "ர்" in letters and letters.index("ர்") != len(letters) - 1:
        return meymayakkam_checker(
            letters, "ர்", ["க்", "ங்", "ச்", "ஞ்", "த்", "ந்", "ப்", "ம்", "ய்", "வ்"]
        )
    else:
        return None


# "மெய்ம்மயக்கம்13": "ழ்+கசதபஞநமயவங"
def meymayakkam13(word):
    """
    meymayakkam13 is the implementation of Tholkappiyar rule of mei mayakkam for the letter "ழ்"

    The Letter  "ழ்" must be followed by any derivatives of "க்","ங்","ச்","ஞ்","த்","ந்","ப்","ம்","ய்","வ்" (i.e) க, கா,	கி... (or) ங,ஙா,ஙி,... (or) ச,சா ,சி... (or) ஞ ,ஞா,ஞி... (or) த,	த,,	தி... (or) ந,	நா,	நி... (or) ப,	பா,	பி... (or) ம,	மா,	மி ... (or) ய,	யா,	யி... (or) வ,	வா,	வி...

        Parameters:
            word (str) : word you want to check the grammar correctness

        Returns :
            result (bool):
                True - The rule matches and it it grammatically correct for "ழ்"
                False - The rule matches and it is not correct for the letter "ழ்"
                None - The rule doesn't apply since there is no "ழ்" in the word
    """
    letters = tamil.utf8.get_letters(word)
    if "ழ்" in letters and letters.index("ழ்") != len(letters) - 1:
        return meymayakkam_checker(
            letters, "ழ்", ["க்", "ங்", "ச்", "ஞ்", "த்", "ந்", "ப்", "ம்", "ய்", "வ்"]
        )
    else:
        return None


# "மெய்ம்மயக்கம்14": "வ்+வ"
def meymayakkam14(word):
    """
    meymayakkam14 is the implementation of Tholkappiyar rule of mei mayakkam for the letter "வ்"

    The Letter  "வ்" must be followed by any derivatives of "வ்" (i.e) வ,	வா,	வி...

        Parameters:
            word (str) : word you want to check the grammar correctness

        Returns :
            result (bool):
                True - The rule matches and it it grammatically correct for "வ்"
                False - The rule matches and it is not correct for the letter "வ்"
                None - The rule doesn't apply since there is no "வ்" in the word
    """
    letters = tamil.utf8.get_letters(word)
    if "வ்" in letters and letters.index("வ்") != len(letters) - 1:
        return meymayakkam_checker(letters, "வ்", ["வ்"])
    else:
        return None


# "மெய்ம்மயக்கம்15": "ல்+கசபலயவ"
def meymayakkam15(word):
    """
    meymayakkam15 is the implementation of Tholkappiyar rule of mei mayakkam for the letter "ல்"

    The Letter  "ல்" must be followed by any derivatives of "க்","ச்","ப்","ல்","ய்","வ்" (i.e) க, கா,	கி... (or) ச,சா ,சி... (or) ப,	பா,	பி... (or) ல,	லா,	லி... (or) ய,	யா,	யி... (or) வ,	வா,	வி...

        Parameters:
            word (str) : word you want to check the grammar correctness

        Returns :
            result (bool):
                True - The rule matches and it it grammatically correct for "ல்"
                False - The rule matches and it is not correct for the letter "ல்"
                None - The rule doesn't apply since there is no "ல்" in the word

    """
    letters = tamil.utf8.get_letters(word)
    if "ல்" in letters and letters.index("ல்") != len(letters) - 1:
        return meymayakkam_checker(letters, "ல்", ["க்", "ச்", "ப்", "ல்", "ய்", "வ்"])
    else:
        return None


# "மெய்ம்மயக்கம்16": "ள்+கசபளயவ"
def meymayakkam16(word):
    """
    meymayakkam16 is the implementation of Tholkappiyar rule of mei mayakkam for the letter "ள்"

    The Letter  "ள்" must be followed by any derivatives of "க்","ச்","ப்","ள்","ய்","வ்" (i.e) க, கா,	கி... (or) ச,சா ,சி... (or) ப,	பா,	பி... (or) ள,	ளா,	ளி... (or) ய,	யா,	யி... (or) வ,	வா,	வி...

        Parameters:
            word (str) : word you want to check the grammar correctness

        Returns :
            result (bool):
                True - The rule matches and it it grammatically correct for "ள்"
                False - The rule matches and it is not correct for the letter "ள்"
                None - The rule doesn't apply since there is no "ள்" in the word

    """
    letters = tamil.utf8.get_letters(word)
    if "ள்" in letters and letters.index("ள்") != len(letters) - 1:
        return meymayakkam_checker(letters, "ள்", ["க்", "ச்", "ப்", "ள்", "ய்", "வ்"])
    else:
        return None


# "மெய்ம்மயக்கம்17": "ற்+கசபற"
def meymayakkam17(word):
    """
    meymayakkam17 is the implementation of Tholkappiyar rule of mei mayakkam for the letter "ற்"

    The Letter  "ற்" must be followed by any derivatives of "க்","ச்","ப்","ற்" (i.e) க, கா,	கி... (or) ச,சா ,சி... (or) ப,	பா,	பி... (or) ற,	றா,	றி...

        Parameters:
            word (str) : word you want to check the grammar correctness

        Returns :
            result (bool):
                True - The rule matches and it it grammatically correct for "ற்"
                False - The rule matches and it is not correct for the letter "ற்"
                None - The rule doesn't apply since there is no "ற்" in the word
    """
    letters = tamil.utf8.get_letters(word)
    if "ற்" in letters and letters.index("ற்") != len(letters) - 1:
        return meymayakkam_checker(letters, "ற்", ["க்", "ச்", "ப்", "ற்"])
    else:
        return None


# "மெய்ம்மயக்கம்18": "ன்+கசஞபமயவறன"
def meymayakkam18(word):
    """
    meymayakkam18 is the implementation of Tholkappiyar rule of mei mayakkam for the letter "ன்"

    The Letter  "ன்" must be followed by any derivatives of "க்","ச்","ஞ்","ப்","ம்","ய்","வ்","ற்","ன்" (i.e) க, கா,	கி... (or) ச,சா ,சி... (or) ஞ ,ஞா,ஞி... (or) ப,	பா,	பி... (or) ம,	மா,	மி ... (or) ய,	யா,	யி... (or) வ,	வா,	வி... (or) ற,	றா,	றி... (or) ன,	னா,	னி...

        Parameters:
            word (str) : word you want to check the grammar correctness

        Returns :
            result (bool):
                True - The rule matches and it it grammatically correct for "ன்"
                False - The rule matches and it is not correct for the letter "ன்"
                None - The rule doesn't apply since there is no "ன்" in the word
    """
    letters = tamil.utf8.get_letters(word)
    if "ன்" in letters and letters.index("ன்") != len(letters) - 1:
        return meymayakkam_checker(
            letters, "ன்", ["க்", "ச்", "ஞ்", "ப்", "ம்", "ய்", "வ்", "ற்", "ன்"]
        )
    else:
        return None



import unittest
from tamilrulepy.mozhimarabu import word_ending

 
class TestWordEnding(unittest.TestCase):
    def test_uyir_check(self):
        data_list = [
            'ஆன',
            'புறா',
            'மணி',
            ' ஈ',
            'எறும்பு',
            'செரூ',
            'மழையே',
            'சந்தை',
            'அணல்பொ',
            'கப்பலோ',
            "வெள்ளோஔ",
            # false case
            'செவ்',
            'பாழ்',
        ]
        expected = [
        True,True,True,True,True,
        True,True,True,True,True,
        True, False,False ]
        res = map(word_ending.uyir_check,data_list)
        actual = list(res)
        self.assertEqual(expected,actual)


    def test_mellinam_check(self):
        data_list = [
            'கணஞ்',
            'மண்',
            'மனந்',
            'மரம்',
            'மாணவன்',
            # false case
            'அணல்',
            'மயில்',
        ]
        expected = [
        True,True,True,True,True,
        False,False
            ]
        res = map(word_ending.mellinam_check,data_list)
        actual = list(res)
        self.assertEqual(expected,actual)


    def test_idaiyinam_check(self):
        data_list = [
            'வாய்',
            'நடுவர்',
            'கடல்',
            'செவ்',
            'பாழ்',
            'நிழள்',
            # false case
            'மரம்',
            'கணம்',
        ]
        expected = [
        True,True,True,True,True,
        True,False,False
            ]
        res = map(word_ending.idaiyinam_check,data_list)
        actual = list(res)
        self.assertEqual(expected,actual)

    def test_oorezhuthoorumozhi_check(self):
        data_list =  [
            "ஆ",
            "ஈ",
            "ஊ",
            "ஏ",
            "ஐ",
            "ஓ",
            # false case
            'செவ்',
            'பாழ்',
        ]
        expected = [
        True,True,True,True,True,True,False,False
            ]
        res = map(word_ending.oorezhuthoorumozhi_check,data_list)
        actual = list(res)
        self.assertEqual(expected,actual)


    def test_alapedai_check(self):
        data_list =  [
            "அ",
            "இ",
            "உ",
            "எ",
            "ஒ",
            # false case
            "ஆ",
            "ஈ", 
        ]
        expected = [
        True,True,True,True,True,False,False
            ]
        res = map(word_ending.alapedai_check,data_list)
        actual = list(res)
        self.assertEqual(expected,actual)


    def test_suttu_check(self):
        data_list =  [
            "அப்பையன்",
            "இவ்வீடு",
            "இக்காட்சி",
            # false case
            "அவை",
            "அது",
            "அம்மலை",
            "இந்நறுமணம்"
        ]
        expected = [
        True,True,True,False,False,False,False
            ]
        res = map(word_ending.suttu_check,data_list)
        actual = list(res)
        self.assertEqual(expected,actual)

    def test_vinaa_check(self):
        data_list =  [
            "எங்கே",
            "எப்படி",
            "எவ்வாறு",
            # false case
            "என்ன",
            "ஏன்",
            "எதற்கு",
            "இந்நறுமணம்"
        ]
        expected = [
        True,True,True,
        False,False,False,False
            ]
        res = map(word_ending.vinaa_check,data_list)
        actual = list(res)
        self.assertEqual(expected,actual)


 


from tamilrulepy.utils.letters import get_first_letter, get_last_letter, get_root_letter
from tamilrulepy.utils.rules import is_proper_word_ending, is_proper_word_starting
import tamil


def thogai_1(first_word: str, second_word: str) -> str:
    """
    Forms a compound Tamil word (தொகை) by joining two given Tamil words
    based on specific phonetic rules.

    This function checks the following conditions:

    #. The last letter of ``first_word`` is a valid uyirmei (உயிர்மெய்) letter.
    #. The root of the first letter of ``second_word`` is one of
       {``க்``, ``ச்``, ``த்``, ``ப்``}.

    If both conditions are met, the function inserts the corresponding mei (மெய்)
    consonant between the words using a predefined mapping and returns the
    combined result.

    :param first_word: The first Tamil word.
    :type first_word: str
    :param second_word: The second Tamil word.
    :type second_word: str
    :return: A compound Tamil word formed by joining ``first_word`` and ``second_word``
             according to Tamil phonetic rules, or ``None`` if the conditions are not met.
    :rtype: str

    :raises None: This function does not raise exceptions directly.

    **Example:**

    >>> thogai_1("அழகு", "பூ")
    'அழகும் பூ'

    **Notes:**
        - Depends on helper functions: ``get_last_letter``, ``get_first_letter``, and ``get_root_letter``.
        - Uses ``tamil.utf8.uyirmei_letters`` for letter validation.
    """
    fw_last_letter = get_last_letter(first_word)
    if fw_last_letter not in tamil.utf8.uyirmei_letters:
        return None
    sw_first_letter = get_root_letter(get_first_letter(second_word))
    if sw_first_letter not in ("க்", "ச்", "த்", "ப்"):
        return None
    mei_map = {"க்": "ங்", "ச்": "ஞ்", "த்": "ந்", "ப்": "ம்"}
    return first_word + mei_map.get(sw_first_letter) + second_word


def thogai_2(first_word: str, second_word: str) -> str:
    """
    Joins two Tamil words to form a compound expression (தொகை)
    when the first word ends properly as per Tamil grammar rules.

    This function checks the following conditions:

    #. The ``first_word`` must end correctly according to Tamil grammatical rules.
       If not, a ``ValueError`` is raised.
    #. The root of the first letter of ``second_word`` must be one of
       {``ஞ்``, ``ந்``, ``ம்``, ``ய்``, ``வ்``}.

    If both conditions are satisfied, the two words are joined with a space
    and returned as a single string.

    :param first_word: The first Tamil word to be validated and joined.
    :type first_word: str
    :param second_word: The second Tamil word to be appended.
    :type second_word: str
    :return: The combined Tamil expression if valid, otherwise ``None``.
    :rtype: str

    :raises ValueError: If the ending of ``first_word`` is not proper as per grammar.

    **Example:**

    >>> thogai_2("அவன்", "வீடு")
    'அவன் வீடு'

    **Notes:**
        - Uses helper functions: ``is_proper_word_ending``, ``get_first_letter``, and ``get_root_letter``.
        - This function enforces Tamil grammatical correctness for word joining.
    """
    if not is_proper_word_ending(first_word):
        raise ValueError("First word ending is not proper as per grammar")
    if get_root_letter(get_first_letter(second_word)) not in ("ஞ்", "ந்", "ம்", "ய்", "வ்"):
        return None
    return first_word + " " + second_word


def thogai_3(first_word: str, second_word: str) -> str:
    """
    Forms a compound Tamil word (தொகை) by joining two words,
    inserting the root consonant of the second word between them
    when the first word ends properly as per Tamil grammar.

    This function checks the following conditions:

    #. The ``first_word`` must end correctly according to Tamil grammatical rules.
       If not, a ``ValueError`` is raised.
    #. The root of the first letter of ``second_word`` must be one of
       {``ஞ்``, ``ந்``, ``ம்``}.

    If both conditions are satisfied, the function inserts the root consonant
    of ``second_word`` between the two words and returns the combined form.

    :param first_word: The first Tamil word to be validated and joined.
    :type first_word: str
    :param second_word: The second Tamil word whose root consonant may be inserted.
    :type second_word: str
    :return: The compound Tamil word formed by merging the two words
             with the appropriate consonant, or ``None`` if the rules do not match.
    :rtype: str

    :raises ValueError: If the ending of ``first_word`` is not proper as per grammar.

    **Example:**

    >>> thogai_3("அவன்", "மனைவி")
    'அவன்மனைவி'

    **Notes:**
        - Uses helper functions: ``is_proper_word_ending``, ``get_first_letter``, and ``get_root_letter``.
        - Enforces Tamil grammatical correctness before performing word joining.
    """
    if not is_proper_word_ending(first_word):
        raise ValueError("First word ending is not proper as per grammar")

    rl_second = get_root_letter(get_first_letter(second_word))
    if rl_second not in ("ஞ்", "ந்", "ம்"):
        return None
    return first_word + rl_second + second_word


def thogai_4(first_word: str, second_word: str) -> str:
    """
    Forms a compound Tamil expression (தொகை) by applying a phonetic transformation
    between two Tamil words when specific grammatical and phonetic conditions are met.

    This function checks the following conditions:

    #. The last letter of ``first_word`` must be a mei (மெய்) letter or its root consonant.
    #. The root consonant of ``first_word`` must be either ``ண்`` or ``ன்``.
    #. The first letter of ``second_word`` must be either ``ஞா`` or ``யா``.
       (Future work: ``second_word`` should ideally be a verb.)

    If the conditions are satisfied, the function performs a phonetic interchange
    (``பொலி மாற்றம்``) between ``ஞா`` and ``யா`` using a predefined mapping and returns
    the modified combined word.

    :param first_word: The first Tamil word whose ending determines the phonetic rule.
    :type first_word: str
    :param second_word: The second Tamil word, which may undergo a phonetic transformation.
    :type second_word: str
    :return: The transformed compound Tamil word if all conditions are met,
             otherwise ``None``.
    :rtype: str

    :raises None: This function does not raise exceptions directly.

    **Example:**

    >>> thogai_4("தேன்", "யானை")
    'தேன்ஞானை'

    **Notes:**
        - Uses helper functions: ``get_last_letter``, ``get_first_letter``,
          and ``get_root_letter``.
        - Depends on ``tamil.utf8.mei_letters`` for validation.

    TODO::
        This function is **not fully complete**.
        It currently assumes ``second_word`` may start with ``ஞா`` or ``யா``,
        but future enhancement is required to ensure that ``second_word``
        is a verb as per Tamil grammatical rules.

    """
    if get_last_letter(first_word) not in tamil.utf8.mei_letters:
        first_word_last_mei_char = get_root_letter(get_last_letter(first_word))
    else:
        first_word_last_mei_char = get_last_letter(first_word)
    if first_word_last_mei_char not in ("ண்", "ன்"):
        return None
    fl_second = get_first_letter(second_word)
    if fl_second not in (
        "ஞா",
        "யா",
    ):  # TODO: second word needs to be a verb (future work)
        return None
    poli_map = {"ஞா": "யா", "யா": "ஞா"}
    changed_second = poli_map[fl_second] + "".join(
        tamil.utf8.get_letters(second_word)[1:]
    )
    return first_word + changed_second


def thogai_5(first_word: str, second_word: str) -> str:
    """
    Forms a compound Tamil word (தொகை) by joining two words directly
    when the first word ends and the second word begins properly
    according to Tamil grammatical rules.

    This function checks the following conditions:

    #. The ``second_word`` must begin correctly as per Tamil grammar.
       If not, a ``ValueError`` is raised.
    #. The last letter of ``first_word`` must be a mei (மெய்) letter
       or its root consonant.
    #. The root consonant of ``first_word`` must be either ``ண்`` or ``ன்``.

    If all conditions are satisfied, the two words are concatenated directly
    without any phonetic transformation and returned as a combined Tamil word.

    :param first_word: The first Tamil word whose ending is validated before joining.
    :type first_word: str
    :param second_word: The second Tamil word whose beginning must be proper.
    :type second_word: str
    :return: The concatenated Tamil compound word if all grammatical conditions are met,
             otherwise ``None``.
    :rtype: str

    :raises ValueError: If ``second_word`` is not a proper Tamil word.

    **Example:**

    >>> thogai_5("அவன்", "நண்பன்")
    'அவன்னண்பன்'

    **Notes:**
        - Uses helper functions: ``is_proper_word_starting``, ``get_last_letter``,
          and ``get_root_letter``.
        - Depends on ``tamil.utf8.mei_letters`` for identifying consonant endings.
        - No intermediate phonetic changes are applied in this rule.
    """
    if not is_proper_word_starting(second_word):
        raise ValueError("Second word is not a proper tamil word")
    if get_last_letter(first_word) not in tamil.utf8.mei_letters:
        first_word_last_mei_char = get_root_letter(get_last_letter(first_word))
    else:
        first_word_last_mei_char = get_last_letter(first_word)
    if first_word_last_mei_char not in ("ண்", "ன்"):
        return None
    return first_word + second_word


def thogai_6(first_word: str, second_word: str) -> str:
    """
    Forms a compound Tamil word (தொகை) by joining two words directly
    when the first word ends and the second word begins properly
    according to following thogaimarabu grammatical rules.

    1. When first_word ends with the following letters ``ன்``,``ண்`` then the joins will be normal joins
    2. When second_word starts with (வல்லினம்) vallinam letter second words first letter will be disort "ன்" |rarr| "ற்", "ண்" |rarr| "ட்" and join

    :param first_word: The first Tamil word whose ending is validated before joining
    :type first_word: str
    :param second_word: The second Tamil word whose beginning must be proper.
    :type second_word: str
    :return: The concatenated Tamil compound word if all grammatical conditions are met,
             otherwise ``None``.
    :rtype: str

    :raises ValueError: If ``second_word`` is not a proper Tamil word.

    **Example:**

    >>> thogai_6("பொன்","குடம்")
    'பொற்குடம்'

    **Notes:**
        - Uses helper functions: ``get_first_letter``, ``get_last_letter``,
          and ``get_root_letter``.
        - Depends on ``tamil.utf8.vallinam_letters`` for identifying second word starting.
        - No intermediate phonetic changes are applied in this rule.
    """
    if get_root_letter(get_first_letter(second_word)) in tamil.utf8.vallinam_letters:
        first_word_last_mei_char = get_last_letter(first_word)
        letter_map = {"ன்": "ற்", "ண்": "ட்"}
        return (
            "".join(tamil.utf8.get_letters(first_word)[:-1])
            + letter_map[first_word_last_mei_char]
            + second_word
        )
    else:
        return first_word + second_word


def thogai_7(first_word: str, second_word: str) -> str:
    """
    Forms a compound Tamil word (தொகை) by joining two words directly
    when the first word ends and the second word begins properly
    according to following thogaimarabu grammatical rules.

    1. when first word ends with ல், ன் second word starts with த, ந then second word's starting and first words ending changes into ற, ன

    :param first_word: The first Tamil word whose ending is validated before joining.
    :type first_word: str
    :param second_word: The second Tamil word whose beginning must be proper.
    :type second_word: str
    :return: The concatenated Tamil compound word if all grammatical conditions are met,
             otherwise ``None``.
    :rtype: str

    :raises ValueError: If ``second_word`` is not a proper Tamil word.

    **Example:**

    >>> thogai_6("பொன்","குடம்")
    'பொற்குடம்'

    **Notes:**
        - Uses helper functions: ``get_first_letter``, ``get_last_letter``,
          and ``get_root_letter``.
        - Depends on ``tamil.utf8.vallinam_letters`` for identifying second word starting.
        - No intermediate phonetic changes are applied in this rule.

    """
    second_word_first_letter = get_first_letter(second_word)
    second_word_first_letters_mei = get_root_letter(second_word_first_letter)
    if get_last_letter(first_word) in ["ல்", "ன்"] and second_word_first_letters_mei in [
        "த்",
        "ந்",
    ]:
        letter_map = {"த்": "ற்", "ந்": "ன்"}
        new_letter = tamil.utf8.join_letters_elementary(
            [
                letter_map[second_word_first_letters_mei],
                tamil.utf8.splitMeiUyir(second_word_first_letter)[1],
            ]
        )
        return (
            "".join(tamil.utf8.get_letters(first_word)[:-1])
            + letter_map[second_word_first_letters_mei]
            + new_letter
            + "".join(tamil.utf8.get_letters(second_word)[1:])
        )
    else:
        return ""


def thogai_8(first_word: str, second_word: str) -> str:
    """
    1. when first_word ends with ண்,ள் and second word starts with த, ந then resulting joining letter turns into ட, ண letters
    """
    second_word_first_letter = get_first_letter(second_word)
    second_word_first_letter_mei = get_root_letter(second_word_first_letter)
    if get_last_letter(first_word) in ["ண்", "ள்"] and second_word_first_letter_mei in [
        "த்",
        "ந்",
    ]:
        letter_map = {"த்": "ட்", "ந்": "ண்"}
        new_letter = tamil.utf8.join_letters_elementary(
            [
                letter_map[second_word_first_letter_mei],
                tamil.utf8.splitMeiUyir(second_word_first_letter)[1],
            ]
        )
        return (
            "".join(tamil.utf8.get_letters(first_word)[:-1])
            + letter_map[second_word_first_letter_mei]
            + new_letter
            + "".join(tamil.utf8.get_letters(second_word)[1:])
        )
    else:
        return ""


def thogai_9(first_word: str, second_word: str) -> str:
    """
    1.
    """


def thogai_10(first_word: str, second_word: str) -> str:
    """
    1.
    """


def thogai_11(first_word: str, second_word: str) -> str:
    """
    1.
    """


def thogai_12(first_word: str, second_word: str) -> str:
    """ """


def thogai_13(first_word: str, second_word: str) -> str:
    """ """


def thogai_14(first_word: str, second_word: str) -> str:
    """ """


def thogai_15(first_word: str, second_word: str) -> str:
    """ """



import unittest
from tamilrulepy.euphonic import get 

 
class Test_Uyirmayaṅkiyal(unittest.TestCase):

    def test_mayaṅkiya_rule_1(self):
        #result = get(["அ","மடம்"])
        #self.assertIn(['அம்மடம்'],result)
        pass

    """
    def test_mayaṅkiya_rule_2(self):
        result = get(["உண","கொண்டான்"])
        self.assertIn(['உணக்கொண்டான்'],result) 
        result = get( ["புலிபோல","கொண்டான்"])
        self.assertIn(["புலிபோலக்கொண்டான்"],result) 
        result = get(["கொள்ளென","கொண்டான்"])
        self.assertIn(['கொள்ளெனக்கொண்டான்'],result) 
        result = get( ["அ","கொண்டான்"])
        self.assertIn(["அக்கொண்டான்"],result) 
        result = get(["ஆங்க","கொண்டான்"])
        self.assertIn(['ஆங்கக்கொண்டான்'],result)  
    
    
    def test_mayaṅkiya_rule_3(self):
        result = get(["அ","ஞாலம்"])
        self.assertIn(['அஞ்ஞாலம்'],result)
        result = get(["அ","நூல்"])
        self.assertIn(['அந்நூல்'],result)
        result = get(["அ","மடம்"])
        self.assertIn(['அம்மடம்'],result)
   
    def test_mayaṅkiya_rule_5(self):
        result = get(["அ","யாழ்"])
        self.assertIn(['அவ்யாழ்'],result)
        result = get(["அ","வளை"])
        self.assertIn(['அவ்வளை'],result) 

    def test_mayaṅkiya_rule_5(self):
        result = get(["அ","அணி"])
        self.assertIn(['அவ்வணி'],result) 
   
    def test_mayaṅkiya_rule_6(self):
        result = get(["அ","இடை"])
        #self.assertIn(['அயிடை'],result) 
        result = get(["அ","வயின்"])
        #self.assertIn(['ஆவயின்'],result) 

    """



from tamilstring import String
from tamil.utf8 import joinMeiUyir
from . import previous

# from tamilstring.utf8 import make_letter


def relative_constant_for_second_word_ka_sa_tha_pa(data):
    """
    if second word starting comes with any of the constants ['க்','ச்','த்','ப்']` this rule modify
    the first words endings with relative consonents ['ங்','ஞ்','ந்','ம்']
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    changes = False
    updata = []
    if_has_letters = ["க்", "ச்", "த்", "ப்"]
    was_to_change = ["ங்", "ஞ்", "ந்", "ம்"]
    for index, (first_word, second_word) in enumerate(
        zip(data.copy[:-1], data.copy[1:])
    ):
        if (
            first_word.singleton(-1).kind == "COM"
            and second_word.singleton(0).kind == "COM"
            and second_word.singleton(0).consonent in if_has_letters
        ):
            list_index = if_has_letters.index(second_word.singleton(0).consonent)
            if list_index:
                changes = True
                data.copy[index] = String(first_word + was_to_change[list_index])
                updata.append(data.rules_manager("இயல்பாகத்திரியும்"))
    return (changes, updata)


# தொகை-2
def if_words_ending_letters(data):
    """
    #TODO
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    changes = False
    updata = []
    if_has_letters = ["க்", "ச்", "த்", "ப்"]
    was_to_change = ["ங்", "ஞ்", "ந்", "ம்"]
    for index, (first_word, second_word) in enumerate(
        zip(data.copy[:-1], data.copy[1:])
    ):
        if (
            second_word.singleton(0).kind == "COM"
            and first_word.singleton(-1).vowel == "உ"
            and second_word.singleton(0).consonent in if_has_letters
        ):
            list_index = if_has_letters.index(second_word.singleton(0).consonent)
            if list_index:
                changes = True
                data.copy[index] = String(first_word + was_to_change(list_index))
                updata.append(data.rules_manager("இயல்பாகத்திரியும்"))
    return (changes, updata)


# தொகை-3
def ஞ்ந்ம்(data):
    """
    first_word_ending_na_na_constants
    # TODO
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    changes = False
    updata = []
    if_has_letters = ["ஞ்", "ந்", "ம்"]
    for index, (first_word, second_word) in enumerate(
        zip(data.copy[:-1], data.copy[1:])
    ):
        if (
            second_word.singleton(0).kind == "COM"
            and first_word.singleton(-1).vowel == "உ"
            and second_word.singleton(0).consonent in if_has_letters
        ):
            list_index = if_has_letters.index(second_word.singleton(0).consonent)
            if list_index:
                changes = True
                data.copy[index] = String(
                    first_word + second_word.singleton(0).consonent
                )
                updata.append(data.rules_manager("இயல்பாகத்திரியும்"))
    return (changes, updata)


# தொகை - 4
def first_word_ending_na3_na5_consonents(data):
    """
    if first word comes with any of the constants ['ன்','ண்'] at the ending then it will try to match other rules such as
    1) if second word starts with this consonents ['ய்','ஞ்'] then it will convert the `ய்` into `ஞ்`, no need to converts `ஞ்` into again it
    2) if second word starts with ["க்", "ச்", "ட்", "த்", "ப்", "ற்"] then it converts `ண்` into `ட்`
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    changes = False
    updata = []
    for index, (first_word, second_word) in enumerate(
        zip(data.copy[:-1], data.copy[1:])
    ):
        if first_word.singleton(-1).consonent in ["ன்", "ண்"]:
            if second_word.singleton(0).consonent in ["ய்", "ஞ்"]:
                changes = True
                second_word[0] = joinMeiUyir("ஞ்", second_word.singleton(0).vowel)
                data.copy[index + 1] = second_word
                updata.append(data.rules_manager("இயல்பாகத்திரியும்"))

            if second_word.singleton(0).consonent in ["க்", "ச்", "ட்", "த்", "ப்", "ற்"]:
                changes = True
                first_word[-1] = (
                    "ட்" if first_word.singleton(-1).consonent == "ண்" else "ற்"
                )
                data.copy[index + 1] = second_word
                updata.append(data.rules_manager("இயல்பாகத்திரியும்"))

    return (changes, updata)


# தொகை - 5
def first_word_ending_la1_na5_consonents(data):
    """
    if first word comes with any of the constants ['ல்','ன்'] at the ending then it will make some modifications are
    'ந்' into 'ன்' and 'த்' into 'ற்'(it does not stop with  'த்' into 'ற்' also converts first words ending consonents into 'ற்')
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    changes = False
    updata = []
    for index, (first_word, second_word) in enumerate(
        zip(data.copy[:-1], data.copy[1:])
    ):
        if first_word.singleton(-1).consonent in ["ல்", "ன்"]:
            if second_word.singleton(0).consonent in ["ந்"]:
                changes = True
                second_word[0] = joinMeiUyir("ன்", second_word.singleton(0).vowel)
                data.copy[index + 1] = second_word
                updata.append(data.rules_manager("இயல்பாகத்திரியும்"))
            if second_word.singleton(0).consonent in ["த்"]:
                changes = True
                first_word[-1] = "ற்"
                second_word[0] = joinMeiUyir("ற்", second_word.singleton(0).vowel)
                data.copy[index + 1] = second_word
                updata.append(data.rules_manager("இயல்பாகத்திரியும்"))
    return (changes, updata)


def apply(data):
    """
    it apply all the particals(சாரியை) rules and give the result of the words
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """

    status = []

    previous_apply = previous.apply(data)
    if len(previous_apply) > 0:
        status.append(previous_apply[1])

    grammar_methods = [
        relative_constant_for_second_word_ka_sa_tha_pa,
        first_word_ending_na3_na5_consonents,
        first_word_ending_la1_na5_consonents,
    ]
    for grammar in grammar_methods:
        return_responce = grammar(data)
        if return_responce[0] == True:
            status.append(return_responce[1])

    return status



from tamilstring import String


def add_joining_consonents(data):
    """
    it will add if required joining consonents letter in between two words (ய்,வ்)
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    changes = False
    for index, (first_word, second_word) in enumerate(
        zip(data.copy[:-1], data.copy[1:])
    ):
        if second_word.singleton(0).kind == "VOL" and second_word.string != "ஐ":
            if (
                first_word.singleton(-1).kind == "VOL"
                or first_word.singleton(-1).kind == "COM"
            ):
                changes = True
                if second_word.singleton(0).letter in ["இ", "ஈ", "ஐ"]:
                    data.copy[index] = String(first_word + "வ்")
                else:
                    data.copy[index] = String(first_word + "ய்")
                data.rule = "add_joining_consonents"
    return (changes, data.status())


def make_double_required_ending_consonents(data):
    """
    it makes double if ending consonents letter is ற் or ட்
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    changes = False
    for index, (first_word, second_word) in enumerate(
        zip(data.copy[:-1], data.copy[1:])
    ):
        if (
            second_word.singleton(0).kind == "VOL"
            and first_word.singleton(-1).kind == "CON"
        ):
            if first_word.singleton(-1).letter in ["ற்", "ட்"]:
                # TODO
                if first_word.singleton(-2).kind == "CON":
                    continue
                else:
                    changes = True
                    data.copy[index] = String(
                        first_word + first_word.singleton(-1).letter
                    )
                    data.rule = "make_double_required_ending_consonents"
    return (changes, data.status())


def adding_relative_consonents(data):
    """
    it adds the relative consonents (ங்,ஞ்,ண்,ந்,ம்,ன்) letter for this consonents (க்,ச்,ட்,த்,ப்,ற்)
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    letters = ["க்", "ச்", "ட்", "த்", "ப்", "ற்"]
    related_letter = ["ங்", "ஞ்", "ண்", "ந்", "ம்", "ன்"]
    changes = False
    for index, (first_word, second_word) in enumerate(
        zip(data.copy[:-1], data.copy[1:])
    ):
        if (
            first_word.singleton(-1).kind == "COM"
            and second_word.singleton(0).kind == "COM"
            or first_word.singleton(-1).kind == "VOL"
            and second_word.singleton(0).kind == "COM"
        ):
            if second_word.singleton(0).consonent in letters:
                changes = True
                data.copy[index] = String(
                    first_word
                    + related_letter[
                        letters.index(data.words.next_word.singleton(0).consonent)
                    ]
                )
                data.rule = "adding_relative_consonents"
    return (changes, data.status())


def adding_ka_sa_tha_pa_consonents(data):
    """
    it adds the required excess consonents (க்,ச்,ப்) for containing this (க்,ச்,ப்) composite letters
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    changes = False
    for index, (first_word, second_word) in enumerate(
        zip(data.copy[:-1], data.copy[1:])
    ):
        if (
            first_word.singleton(-1).kind == "COM"
            and second_word.singleton(0).kind == "COM"
            or first_word.singleton(-1).kind == "VOL"
            and second_word.singleton(0).kind == "COM"
        ):
            if second_word.singleton(0).consonent in ["க்", "ச்", "ப்"]:
                changes = True
                data.copy[index] = String(
                    first_word + second_word.singleton(0).consonent
                )
                data.rule = "adding_ka_sa_tha_pa_consonents"
    return (changes, data.status())


def remove_ending_consonent_u(data):
    """
    it removes the vowel (உ) from the ending of the word that containing composite letters
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    changes = False
    for index, (first_word, second_word) in enumerate(
        zip(data.copy[:-1], data.copy[1:])
    ):
        if second_word.singleton(0).kind == "VOL":
            if (
                first_word.singleton(-1).kind == "COM"
                and first_word.singleton(-1).vowel == "உ"
            ):
                if len(first_word.letters) >= 2:
                    changes = True
                    if (
                        first_word.singleton(-2).consonent
                        == first_word.singleton(-1).consonent
                        and first_word.singleton(-1).consonent == "த்"
                    ):
                        data.copy[index] = String(first_word[:-1])
                    else:
                        data.copy[index] = String(first_word - "உ")
                    data.rule = "remove_ending_consonent_u"
                else:
                    continue
    return (changes, data.status())


def joining_consonents_with_vowel(data):
    """
    it joins the consonents with vowel in required concardinating two words
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    final_word = None
    for index, (first_word, second_word) in enumerate(
        zip(data.copy[:-1], data.copy[1:])
    ):
        final_word = (
            String(first_word + second_word)
            if final_word == None
            else String(final_word + second_word)
        )
        data.rule = "joining_consonents_with_vowel"
    data.copy = [final_word]
    return (True, data.status())


def apply(data):
    """
    it apply all the common rules and give the result of the words
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    status = []

    grammar_methods = [
        remove_ending_consonent_u,
        make_double_required_ending_consonents,
        adding_ka_sa_tha_pa_consonents,
        add_joining_consonents,
        joining_consonents_with_vowel,
    ]

    for grammar in grammar_methods:
        return_responce = grammar(data)
        if return_responce[0] == True:
            status.append(return_responce[1])

    return status



from typing import List, Dict, Any, Optional
from tamilstring import String
from tamilstring import get_letters

from . import partical, posTag, singleletter, wordsbetween
from . import common
from . import wordsbetween
from . import previous
from . import specificword


class Maintainer:
    def __init__(self, words_list: List[str], lang: "ta") -> None:
        self.list: List[str] = words_list
        self.position: int = 0
        self.src = words_list
        self.copy: List[String] = [String(word) for word in self.list]
        self.rule: Optional[str] = None
        self.lang = lang
        # self.explination: Optional[str] = None

    def status(self) -> Dict[str, Any]:
        return {
            "words": [word.string for word in self.copy],
            "rule": self.rule,
            # "explination": self.explination
        }

    def words_status(self) -> List[str]:
        return [word.string for word in self.copy]

    def restore(self):
        self.copy: List[String] = [String(word) for word in self.src]

    def rules_manager(self, rule):

        status = {"words": self.words_status(), "rule": rule}

        self.restore()
        return status

    @property
    def previous_word(self) -> String:
        return self.copy[self.position - 1]

    @property
    def current_word(self) -> String:
        return self.copy[self.position]

    @property
    def next_word(self) -> String:
        try:
            return self.copy[self.position + 1]
        except IndexError:
            return String("")

    @previous_word.setter
    def previous_word(self, word: str) -> None:
        self.copy[self.position - 1].string = word

    @current_word.setter
    def current_word(self, word: str) -> None:
        self.copy[self.position].string = word

    @next_word.setter
    def next_word(self, word: str) -> None:
        self.copy[self.position + 1].string = word


class WordsGenerator:

    final_list: List[List[Dict[str, Any]]] = []

    def updata_final_list(self, result) -> None:
        self.final_list = result

    def get_words(self):
        return self.final_list

    def grammer_applyer(self, data, msg=[], previous_stage=None):
        """
        Returns :
            dict : containing all the applied grammar rules and modifications
        """
        if len(msg) < 1:

            self.final_list.append(common.apply(data))

        else:
            for each_type in msg:

                if len(each_type) > 1:
                    for each in each_type:
                        self.grammer_orginizer(each, data, previous_stage)
                else:
                    self.grammer_orginizer(each_type[0], data, previous_stage)

        # return self.final_list

    def grammer_orginizer(self, each_type, data, previous_stage):

        data.copy = [String(word) for word in each_type["words"]]

        after = common.apply(data)

        appending_list = [each_type]

        appending_list.extend(after)

        if len(previous_stage) > 0:
            appending_list = previous_stage + appending_list

        appending_list.insert(0, {"words": data.src, "rule": "given words"})

        self.final_list.append(appending_list)

    def config_grammer_rules(self, data: Maintainer):

        self.final_list = []

        len_of_words_list = len(data.list)

        previous_stage = previous.apply(data)

        if len_of_words_list == 1:
            word_letter = get_letters(data.list[0])
            if len(word_letter) == 1:
                self.final_list = singleletter.apply(word_letter[0])
            else:
                self.final_list = [[{"words": data.list[0]}]]

        elif len_of_words_list == 2:
            if data.list[-1] in posTag.Case:
                self.grammer_applyer(data, specificword.apply(data), previous_stage)

            elif data.list[-1] in ["குறை"]:
                self.grammer_applyer(data, specificword.apply(data), previous_stage)

            elif data.list[0] in ["அ", "இ", "எ"]:
                self.grammer_applyer(data, specificword.apply(data), previous_stage)

            elif data.list[0] in posTag.wh_adverb_e_i:
                self.grammer_applyer(data, wordsbetween.apply(data), previous_stage)

            elif data.list[0] in posTag.list_of_words:
                self.grammer_applyer(data, wordsbetween.apply(data), previous_stage)

            else:
                self.grammer_applyer(data, wordsbetween.apply(data), previous_stage)

        else:

            for word in data.list:
                if word in posTag.Particals and data.position < len(data.list) - 1:
                    self.grammer_applyer(data, partical.apply(data), previous_stage)

                data.position += 1



# தொல்காப்பியர்_மொழியிறுதி_விதிகள் (Tholkaappiar_word_ending_rule)
# இந்நிரலாக்கம் தமிழ் மொழித் தரவுகளைத் தொல்காப்பிய விதிகளைக் கொண்டு கணினி வழியாக மொழியிறுதி எழுத்துக்களைக் கண்டறிவதற்கான நிரல் தொகுப்பாகும்.
# இந்நிரலாக்கத்தை உருவாக்கியவர்கள் - அ.பரமேசுவரன், முனைவர் த. சத்தியராஜ் (நேயக்கோ)
import tamil

# பைத்தானில் தமிழ் மொழியை உள்ளீடு செய்வதற்கு இது பயன்படுத்தப்பெற்றுள்ளது.

# இது தரவை உள்ளீடாய்த் தருவதற்குரிய பகுதி்.


def uyir_check(word):
    """
    Checks if the last letter of a given Tamil word is a 'உயிரெழுத்து(Uyir)' (vowel) character.

    This function verifies whether the last letter of the input word is one of the
    following Tamil vowels: அ, ஆ, இ, ஈ, உ, ஊ, ,எ ,ஏ, ஐ, ஒ, ஓ, or ஔ. It checks the root form of the
    last letter to determine if it belongs to the 'Uyir' character set.

    Parameters:
    word (str): The input Tamil word to be checked.

    Returns:
    bool: True if the last letter of the word is a Tamil vowel ('உயிரெழுத்து').
    False: If the last letter is not a Tamil vowel.
    """
    uyir_list = ["அ", "ஆ", "இ", "ஈ", "உ", "ஊ", "எ", "ஏ", "ஐ", "ஒ", "ஓ", "ஔ"]
    last_letter = tamil.utf8.get_letters(word)[-1]
    root_words = tamil.utf8.splitMeiUyir(last_letter)
    if type(root_words) == tuple:
        root_last = root_words[-1]
    else:
        root_last = root_words
    if root_last in uyir_list:
        return True
    else:
        return False


# இது உயிர் எழுத்தில் முடியும் உயிர்மெய் எழுத்துக்களைப் பொருத்திப் பார்ப்பதற்குரிய நிரலாகும்.


def mellinam_check(word):
    """
    Checks if the last letter of a given Tamil word is a 'மெல்லினம் (Mellinam)' (nasal consonant) character.

    This function verifies whether the last letter of the input word is one of the
    following nasal consonants: ஞ், ண், ந், ம், or ன். It checks the root form of the
    last letter to determine if it belongs to the 'மெல்லினம் (Mellinam)' character set.

    Parameters:
    word (str): The input Tamil word to be checked.

    Returns:
    bool: True if the last letter of the word is a 'மெல்லினம் (Mellinam)' character.
    False: If the last letter is not a 'மெல்லினம் (Mellinam)' character.
    """
    mellinam_list = ["ஞ்", "ண்", "ந்", "ம்", "ன்"]
    last_letter = tamil.utf8.get_letters(word)[-1]
    root_words = tamil.utf8.splitMeiUyir(last_letter)
    if type(root_words) == tuple:
        root_last = root_words[-1]
    else:
        root_last = root_words
    if root_last in mellinam_list:
        return True
    else:
        return False


# இது மெல்லின எழுத்துக்களைப் பொருத்திப் பார்ப்பதற்குரிய நிரலாகும்.


def idaiyinam_check(word):
    """
    Checks if the last letter of a given Tamil word is an 'இடையினம் (Idaiyinam)' (semi-vowel) character.

    This function verifies whether the last letter of the input word is one of the
    following semi-vowels: ய் ,ர் ,ல், வ், ழ் or ள். It checks the root form of the
    last letter to determine if it belongs to the 'இடையினம் (Idaiyinam)' character set.

    Parameters:
    word (str): The input Tamil word to be checked.

    Returns:
    bool: True if the last letter of the word is an 'இடையினம் (Idaiyinam)' character.
    False: If the last letter is not an 'இடையினம் (Idaiyinam)' character.
    """
    idaiyinam_list = ["ய்", "ர்", "ல்", "வ்", "ழ்", "ள்"]
    last_letter = tamil.utf8.get_letters(word)[-1]
    root_words = tamil.utf8.splitMeiUyir(last_letter)
    if type(root_words) == tuple:
        root_last = root_words[-1]
    else:
        root_last = root_words
    if root_last in idaiyinam_list:
        return True
    else:
        return False


# இது இடையின எழுத்துக்களைப் பொருத்திப் பார்ப்பதற்குரிய நிரலாகும்.


def oorezhuthoorumozhi_check(word):
    """
    Checks if the last letter of a given Tamil word is an 'ஓரெழுத்து ஒருமொழி (oorezhuthuoorumozhi)' (long vowel) character.

    This function verifies whether the last letter of the input word is one of the
    following long vowels: ஆ, ஈ, ஊ, ஏ, ஐ, or ஓ. It checks the root form of the
    last letter to determine if it belongs to the 'ஓரெழுத்து ஒருமொழி (oorezhuthuoorumozhi)' character set.

    Parameters:
    word (str): The input Tamil word to be checked.

    Returns:
    bool: True if the last letter of the word is an 'ஓரெழுத்து ஒருமொழி (oorezhuthuoorumozhi)' character.
    False: If the last letter is not an 'ஓரெழுத்து ஒருமொழி (oorezhuthuoorumozhi)' character.
    """
    oorezhuthoorumozhi_list = ["ஆ", "ஈ", "ஊ", "ஏ", "ஐ", "ஓ"]
    last_letter = tamil.utf8.get_letters(word)[-1]
    root_words = tamil.utf8.splitMeiUyir(last_letter)
    if type(root_words) == tuple:
        root_last = root_words[-1]
    else:
        root_last = root_words
    if root_last in oorezhuthoorumozhi_list:
        return True
    else:
        return False


# இது ஓரெழுத்தொரு மொழி எழுத்துக்களைப் பொருத்திப் பார்ப்பதற்குரிய நிரலாகும்.


def alapedai_check(word):
    """
    Checks if the last letter of a given Tamil word is an 'அளபெடை (Alapedai)' (short vowel) character.

    This function verifies whether the last letter of the input word is one of the
    following short vowels: அ, இ, உ, எ, or ஒ.

    Parameters:
    word (str): The input Tamil word to be checked.

    Returns:
    bool: True if the last letter of the word is an 'அளபெடை (Alapedai)' character.
    False: If the last letter is not an 'அளபெடை (Alapedai)' character.
    """
    alapedai_list = ["அ", "இ", "உ", "எ", "ஒ"]
    last_letter = tamil.utf8.get_letters(word)[-1]
    if last_letter in alapedai_list:

        return True
    else:
        return False


# இது அளபெடை எழுத்துக்களைப் பொருத்திப் பார்ப்பதற்குரிய நிரலாகும்.


def suttu_check(word):
    """
    Checks if a given Tamil word follows the 'சுட்டெழுத்து(Suttu)' (consonant + vowel) structure.

    This function checks if the first letter is one of the short vowels (அ, இ, உ) and
    the second letter is one of the consonants (க்,ச்,த்,ப்,வ்). It is used to validate
    the 'சுட்டெழுத்து(Suttu)' (consonant + vowel) structure of the word.

    Parameters:
    word (str): The input Tamil word to be checked.

    Returns:
    bool: True if the word starts with a short vowel followed by a consonant.
    False: If the word does not follow the 'சுட்டெழுத்து(Suttu)' structure.
    """
    suttu_list = ["அ", "இ", "உ"]
    suttu_mey_list = ["க்", "ச்", "த்", "ப்", "வ்"]
    first_letter = tamil.utf8.get_letters(word)[0]
    second_letter = tamil.utf8.get_letters(word)[1]

    if first_letter in suttu_list and second_letter in suttu_mey_list:
        return True
    else:
        return False


# இது சுட்டு எழுத்துக்களைப் பொருத்திப் பார்ப்பதற்குரிய நிரலாகும்.


def vinaa_check(word):
    """
    Checks if the first two letters of a given Tamil word form a specific pattern: 'Vinaa'.

    This function checks if the first letter is "எ" and the second letter is one of the
    consonants: க்,ச்,த்,ப்,வ்,ங்,ந்.

    Parameters:
    word (str): The input Tamil word to be checked.

    Returns:
    bool: True if the word follows the 'Vinaa' pattern of 'எ' followed by a consonant.
    False: If the word does not match the 'Vinaa' pattern.
    """
    vinaa_mey_list = ["க்", "ச்", "த்", "ப்", "வ்", "ங்", "ந்"]
    if len(word) < 2:
        return False
    first_letter = tamil.utf8.get_letters(word)[0]
    second_letter = tamil.utf8.get_letters(word)[1]

    if first_letter == "எ" and second_letter in vinaa_mey_list:
        return True
    else:
        return False


# இது வினா எழுத்துக்களைப் பொருத்திப் பார்ப்பதற்குரிய நிரலாகும்.

# இது இறுதியாக அனைத்து விதிகளையும் பொருத்திப் பார்த்து பதில் அளிப்பதற்குரிய நிரலாகும்.



import tamil

from tamilrulepy.mozhimarabu.word_starting import (
    uyirmei_ka_check,
    uyirmei_ma_check,
    uyirmei_na_check,
    uyirmei_nga_check,
    uyirmei_pa_check,
    uyirmei_sa_check,
    uyirmei_ta_check,
    uyirmei_va_check,
    uyirmei_ya_check,
)

from tamilrulepy.mozhimarabu.word_ending import (
    uyir_check,
    mellinam_check,
    idaiyinam_check,
    alapedai_check,
    oorezhuthoorumozhi_check,
    suttu_check,
    vinaa_check,
)

WORD_STARTING_RULES = (
    uyirmei_ka_check,
    uyirmei_ma_check,
    uyirmei_na_check,
    uyirmei_nga_check,
    uyirmei_pa_check,
    uyirmei_sa_check,
    uyirmei_ta_check,
    uyirmei_va_check,
    uyirmei_ya_check,
)

WORD_ENDING_RULES = (
    uyir_check,
    mellinam_check,
    idaiyinam_check,
    alapedai_check,
    oorezhuthoorumozhi_check,
    suttu_check,
    vinaa_check,
)


def is_proper_word_starting(word: str) -> bool:
    results = []
    for rule in WORD_STARTING_RULES:
        result = rule(word)
        if result is None:
            continue
        results.append(result)
    return all(results) and len(results) != 0


def is_proper_word_ending(word: str) -> bool:
    results = []
    for rule in WORD_ENDING_RULES:
        try:
            result = rule(word)
            results.append(result)
        except Exception:
            continue  # TODO: Need to check if one letter word is coming how to handle rule check
    return any(results) and len(results) != 0



import unittest
from tamilrulepy.euphonic.common import  add_joining_consonents, make_double_required_ending_consonents, remove_ending_consonent_u,joining_consonents_with_vowel
from tamilrulepy.euphonic.stackwords import Maintainer


def words_data(words_list):
    return Maintainer(words_list,lang='ta')



class Test_Euphonic_commin(unittest.TestCase):


    def test_add_joining_consonents(self):
        # true case
       
        case1 = add_joining_consonents( words_data(["ஆ","இன்","ஐ"]) )
        self.assertTrue(case1[0])
  
        case2 = add_joining_consonents( words_data([ "மணி" , "அடித்தான்" ]) )
        self.assertTrue(case2[0])
   
        case3 = add_joining_consonents( words_data(["பூ" , "அரும்பு"]) )
        self.assertTrue(case3[0])
  
        case4 = add_joining_consonents( words_data([ "பூ" , "இதழ்" ]) )
        self.assertTrue(case4[0])
 

        # false case


        case11 = add_joining_consonents( words_data(["அவை","வற்று","ஐ"]) )
        self.assertFalse(case11[0])
  
        case12 = add_joining_consonents( words_data([ "பல்" , "இல்லை"  ]) )
        self.assertFalse(case12[0])
   
        case13 = add_joining_consonents( words_data([ "நட்" , "இரவில்" ]) )
        self.assertFalse(case13[0])
  
        case14 = add_joining_consonents( words_data([ "கண்" , "அழகிய" ]) )
        self.assertFalse(case14[0])
        
        case15 = add_joining_consonents( words_data([ 'மரம்' , 'அழகு' ]) )
        self.assertFalse(case15[0])
 

    def test_make_double_required_ending_consonents(self):
        # true case
      
        case1 = make_double_required_ending_consonents( words_data([ 'வீட்','இன்','தென்புறம்' ]) )
        self.assertTrue(case1[0])
  
        case2 = make_double_required_ending_consonents( words_data([ "வெற்" , "இலை" ]) )
        self.assertTrue(case2[0])
   
        case3 = make_double_required_ending_consonents( words_data([ 'நாட்' , 'அரசன்' ]) )
        self.assertTrue(case3[0])
        
        # TODO
        #case4 = make_double_required_ending_consonents( words_data([ 'சோற்' , 'பானை' ]) )
        #self.assertTrue(case4[0])
 

        # false case
        
        case10 = make_double_required_ending_consonents( words_data([ "வெற்ற்" + "இலை" ]) )
        self.assertFalse(case10[0])

        case11 = make_double_required_ending_consonents( words_data([ "மாடு" , "வால்" ]) )
        self.assertFalse(case11[0])
        
        case12 = make_double_required_ending_consonents( words_data([ "ஆ","இன்","ஐ" ]) )
        self.assertFalse(case12[0])
   

    def test_remove_ending_consonent_u(self):
        # true case
     
        case1 = remove_ending_consonent_u( words_data([ 'குன்று','அக்கு','குடி' ]) )
        self.assertTrue(case1[0])
  
        case2 = remove_ending_consonent_u( words_data([ "காடு" , "ஆறு" ]) )
        self.assertTrue(case2[0])
        
        # TODO
        #case3 = remove_ending_consonent_u( words_data(["நாடு" , "யாது"]) )
        #self.assertTrue(case3[0])
        
        case4 = remove_ending_consonent_u( words_data([ "வரகு" , "அடி" ]) )
        self.assertTrue(case4[0])

        case5 = remove_ending_consonent_u( words_data([ "துப்பு" , "அழகு" ]) )
        self.assertTrue(case5[0])
 
        case6 = remove_ending_consonent_u( words_data([ "பட்டு" , "ஆடை" ]) )
        self.assertTrue(case6[0])
  

        # false case
        
        case10 = remove_ending_consonent_u( words_data([ "கு","இன்","ஐ" ]) )
        self.assertFalse(case10[0])

        case11 = remove_ending_consonent_u( words_data([ 'புளி','அம்','நுனி' ]) )
        self.assertFalse(case11[0])
        
        case12 = remove_ending_consonent_u( words_data([ "ஆ","இன்","ஐ" ]) )
        self.assertFalse(case12[0])
   



    #TODO
    #def test_joining_consonents_with_vowel(self):
    #    pass










from typing import Dict, Callable, Any
from .posTag import cardinal_degit, wh_adverb, Case, numbers_in_words


def partical_in(data) -> None:
    """
    it checks the possible words and required word repesentations to make modification around `இன்`
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    changes = False
    updata = []
    if data.previous_word.singleton(-1).vowel == "ஆ":
        changes = True
        updata.append(data.rules_manager("இயல்பாகத்திரியும்"))
        data.current_word = "ன்"
        updata.append(data.rules_manager("இயல்பாகத்திரியும்"))
    if (
        data.previous_word.string in numbers_in_words
        and data.next_word.string in cardinal_degit
        or data.next_word.string in numbers_in_words
    ):
        changes = True
        data.current_word[-1] = "ற்"
        updata.append(data.rules_manager("இயல்பாகத்திரியும்"))

    if data.next_word.string in Case:
        if data.previous_word.string in numbers_in_words:
            pass
        elif data.previous_word.string in numbers_in_words:
            pass

        changes = True
        data.current_word[-1] = "ற்"
        updata.append(data.rules_manager("இயல்பாகத்திரியும்"))

    return (changes, updata)


def partical_varru(data) -> None:
    """
    it checks the possible words and required word repesentations to make modification around `வற்று`
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    changes = False
    updata = []
    if data.previous_word.string in wh_adverb:
        changes = True
        data.current_word[0] = "அ"
        updata.append(data.rules_manager("இயல்பாகத்திரியும்"))

    return (changes, updata)


def partical_an(data) -> None:
    """
    it checks the possible words and required word repesentations to make modification around `ஆன்`
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    changes = True
    updata = []
    data.current_word[-1] = "ற்"
    updata.append(data.rules_manager("இயல்பாகத்திரியும்"))

    return (changes, updata)


def partical_attu(data) -> None:
    """
    it checks the possible words and required word repesentations to make modification around `அத்து`
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    changes = True
    updata = []
    data.current_word[0] = ""
    updata.append(data.rules_manager("இயல்பாகத்திரியும்"))
    return (changes, updata)


def partical_ikku(data) -> None:
    """
    it checks the possible words and required word repesentations to make modification around `இக்கு`
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    changes = True
    updata = []
    data.current_word[0] = ""
    updata.append(data.rules_manager("இயல்பாகத்திரியும்"))
    return (changes, updata)


def partical_akku(data) -> None:
    """
    it checks the possible words and required word repesentations to make modification around `அக்கு`
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    changes = True
    updata = []
    data.current_word = "அ"
    updata.append(data.rules_manager("இயல்பாகத்திரியும்"))
    return (changes, updata)


def partical_am(data) -> None:
    """
    it checks the possible words and required word repesentations to make modification around `அம்`
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    changes = True
    updata = []
    letters = ["க்", "ச்", "த்"]
    related_letter = ["ங்", "ஞ்", "ந்"]
    if data.next_word.singleton(0).consonent in letters:
        changes = True
        data.current_word[-1] = related_letter[
            letters.index(data.next_word.singleton(0).consonent)
        ]
        updata.append(
            data.rules_manager(
                "இயல்பாகத்திரியும்" if data.lang == "en" else "இயல்பாகத்திரியும்"
            )
        )
    else:
        changes = True
        data.current_word[-1] = ""
        updata.append(data.rules_manager("இயல்பாகத்திரியும்"))

    return (changes, updata)


def apply(data):
    """
    it apply all the particals(சாரியை) rules and give the result of the words
    Args:
        data (WordsGenerator): list of words that you want to apply all partical grammar rules
    Return:
        list: [bool,dict]
    """

    grammar_methods = {
        "இன்": partical_in,
        "வற்று": partical_varru,
        "ஆன்": partical_an,
        "அத்து": partical_attu,
        "இக்கு": partical_ikku,
        "அக்கு": partical_akku,
        "அம்": partical_am,
    }

    status = []
    return_responce = grammar_methods[data.current_word.string](data)
    if return_responce[0] == True:
        status.append(return_responce[1])
    return status



import tamil


def get_last_letter(word: str) -> str:
    try:
        return tamil.utf8.get_letters(word)[-1]
    except IndexError:
        return ""


def get_first_letter(word: str) -> str:
    try:
        return tamil.utf8.get_letters(word)[0]
    except IndexError:
        return ""


def get_root_letter(letter: str) -> str:
    return tamil.utf8.splitMeiUyir(letter)[0]



from tamilstring import Letter

phonetic_length_1 = [
    "அ",
    "இ",
    "உ",
    "எ",
    "ஒ",
]

phonetic_length_2 = ["ஆ", "ஈ", "ஊ", "ஏ", "ஐ", "ஓ", "ஔ"]


def apply(letter):
    """
    it apply the particals(சாரியை) rules for the single letters and gives the result of the words
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    original_letter = letter

    return_value = []

    letter = Letter(letter)
    vowel = None

    rule = "எழுத்துச்சாரியை"

    if letter.kind == "VOL":
        vowel = original_letter
    elif letter.kind == "COM":
        consonents_, vowel_ = letter.split_letter
        vowel = vowel_

    if vowel in phonetic_length_2:
        return_value.extend([[{"words": [original_letter + "காரம்"], "rule": rule}]])
    if vowel in phonetic_length_1:
        word_list = [
            original_letter + "காரம்",
            original_letter + "கரம்",
            original_letter + "ஃகான்",
        ]
        return_value.extend([[{"words": [word], "rule": rule}] for word in word_list])
    if vowel in ["ஐ", "ஔ"]:
        return_value.append([{"words": [original_letter + "கான்"], "rule": rule}])

    if len(return_value) == 0:
        return [[{"words": letter}]]
    else:

        return return_value



from tamilstring import String
from . import posTag


def remove_ending_consonent_mai(data):
    """
    it removes `மை` at ending of any word and gives the result of the words
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    changes = False
    for index, (first_word, second_word) in enumerate(
        zip(data.copy[:-1], data.copy[1:])
    ):
        if first_word.singleton(-1).letter == "மை":
            if len(first_word.letters) >= 2:
                changes = True
                data.copy[index] = String(first_word[:-1])
                data.rule = "remove_ending_consonent_mai"
            else:
                continue
    return (changes, data.status())


def remove_ending_consonent_m(data):
    """
    if any last letter of word containing `ம்` having composite or consonent will be removed
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    changes = False
    for index, (first_word, second_word) in enumerate(
        zip(data.copy[:-1], data.copy[1:])
    ):
        if first_word.singleton(-1).consonent == "ம்":
            if (
                len(first_word.letters) >= 2
                and first_word.string not in posTag.Particals
            ):
                changes = True
                data.copy[index] = String(first_word[:-1])
                data.rule = "remove_ending_consonent_m"
            else:
                continue
    return (changes, data.status())


def apply(data):
    """
    it apply all the common rules and give the result of the words
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    status = []

    grammar_methods = [remove_ending_consonent_m, remove_ending_consonent_mai]

    for grammar in grammar_methods:
        return_responce = grammar(data)
        if return_responce[0] == True:
            status.append(return_responce[1])

    if len(status) > 0:
        data.list = status[-1]["words"]

    return status



import unittest
from tamilrulepy.meymayakkam import meymayakkam1, meymayakkam2, meymayakkam3, meymayakkam4, meymayakkam5, meymayakkam6, meymayakkam7, meymayakkam8, meymayakkam9, meymayakkam10, meymayakkam11, meymayakkam12, meymayakkam13, meymayakkam14, meymayakkam15, meymayakkam16, meymayakkam17, meymayakkam18


class TestMeymayakkam(unittest.TestCase):
    
    def test_meymayakkam1(self):
        # correct words
        self.assertTrue(meymayakkam1("மயக்கம்"))
        self.assertTrue(meymayakkam1("மெய்மயக்கம்"))
        # Detecting incorrect words
        self.assertFalse(meymayakkam1("மயக்க்கம்"))
        # Detecting Not Applicable words
        self.assertIsNone(meymayakkam1("தடம்"))
        self.assertIsNone(meymayakkam1("திட்டு"))

    def test_meymayakkam2(self):
        # Correct Words
        self.assertTrue(meymayakkam2("வெங்காயம்"))
        self.assertTrue(meymayakkam2("வேங்கை"))
        self.assertTrue(meymayakkam2("குரங்கு"))
        self.assertTrue(meymayakkam2("அங்குலம்"))
        # Detecting Incorrect words
        self.assertFalse(meymayakkam2("வெங்ங்கயம்"))
        self.assertFalse(meymayakkam2("இனிங்சி"))
        # Detecting Not Applicable words
        self.assertIsNone(meymayakkam2("விளையாட்டு"))
        self.assertIsNone(meymayakkam2("மீன்கள்"))
        self.assertIsNone(meymayakkam2("சாப்பாடு"))

    def test_meymayakkam3(self):
        # Correct Words
        self.assertTrue(meymayakkam3("அச்சரம்"))
        self.assertTrue(meymayakkam3("அச்சாணி"))
        self.assertTrue(meymayakkam3("சிகிச்சை"))
        self.assertTrue(meymayakkam3("சர்ச்சை"))
        # Detecting Incorrect words
        self.assertFalse(meymayakkam3("சட்ச்ச்சி"))
        self.assertFalse(meymayakkam3("திச்டட"))
        # Detecting Not Applicable words
        self.assertIsNone(meymayakkam3("பூங்கா"))
        self.assertIsNone(meymayakkam3("நேரம்"))
        self.assertIsNone(meymayakkam3("நாட்கள்"))


    def test_meymayakkam4(self):
        # Correct Words
        # TODO Still for "ஞ்","ய்" didn't have any words
        self.assertTrue(meymayakkam4("இஞ்சி"))
        self.assertTrue(meymayakkam4("மஞ்சள்"))
        self.assertTrue(meymayakkam4("காஞ்சிபுரம்"))
        self.assertTrue(meymayakkam4("அஞ்சறைப்பெட்டி"))
        # Detecting Incorrect words
        self.assertFalse(meymayakkam4("அஞ்ககி"))
        self.assertFalse(meymayakkam4("மிஞ்வலி"))
        # Detecting Not Applicable words
        self.assertIsNone(meymayakkam4("காலை"))
        self.assertIsNone(meymayakkam4("சிறப்பு"))
        self.assertIsNone(meymayakkam4("வர்ணம்"))

    def test_meymayakkam5(self):
        # Correct Words
        self.assertTrue(meymayakkam5("வெட்கம்"))
        self.assertTrue(meymayakkam5("நுட்பம்"))
        self.assertTrue(meymayakkam5("மாட்சி"))
        self.assertTrue(meymayakkam5("முட்டாள்"))
        # Detecting Incorrect words
        self.assertFalse(meymayakkam5("வெடஂதச"))
        self.assertFalse(meymayakkam5("சாடஂவா"))
        # Detecting Not Applicable words
        self.assertIsNone(meymayakkam5("மஞ்சள்"))
        self.assertIsNone(meymayakkam5("சிறப்பு"))
        self.assertIsNone(meymayakkam5("அச்சரம்"))

    def test_meymayakkam6(self):
        # Correct Words
        self.assertTrue(meymayakkam6("வெண்கலம் "))
        self.assertTrue(meymayakkam6("மண்சேறு"))
        self.assertTrue(meymayakkam6("வெண்ஞமலி"))
        self.assertTrue(meymayakkam6("மண்டலம் "))
        self.assertTrue(meymayakkam6("நண்பகல் "))
        self.assertTrue(meymayakkam6("வெண்மலர்"))
        self.assertTrue(meymayakkam6("மண்யாது"))
        self.assertTrue(meymayakkam6("மண்வலிது"))
        # Detecting Incorrect words
        self.assertFalse(meymayakkam6("கணஂளியே"))
        self.assertFalse(meymayakkam6("சணஂராலய"))
        # Detecting Not Applicable words
        self.assertIsNone(meymayakkam6("நுட்பம்"))
        self.assertIsNone(meymayakkam6("சிகிச்சை"))
        self.assertIsNone(meymayakkam6("காஞ்சிபுரம்"))

    def test_meymayakkam7(self):
        # Correct Words
        self.assertTrue(meymayakkam7("கத்தி"))
        self.assertTrue(meymayakkam7("பத்து"))
        self.assertTrue(meymayakkam7("மத்தளம்"))
        self.assertTrue(meymayakkam7("வித்தகன்"))
        # Detecting Incorrect words
        self.assertFalse(meymayakkam7("சத்நியாச"))
        self.assertFalse(meymayakkam7("பத்சலம்"))
        # Detecting Not Applicable words
        self.assertIsNone(meymayakkam7("நண்பகல் "))
        self.assertIsNone(meymayakkam7("அச்சரம்"))
        self.assertIsNone(meymayakkam7("நண்பகல் "))

    def test_meymayakkam8(self):
        # Correct Words
        self.assertTrue(meymayakkam8("வெந்நீர் "))
        self.assertTrue(meymayakkam8("செந்நீர்"))
        self.assertTrue(meymayakkam8("வந்தான்"))
        self.assertTrue(meymayakkam8("வெரிந்யாது"))
        # Detecting Incorrect words
        self.assertFalse(meymayakkam8("தந்ராலயா"))
        self.assertFalse(meymayakkam8("சிந்ஙாண"))
        # Detecting Not Applicable words
        self.assertIsNone(meymayakkam8("மதஂதமஂ"))
        self.assertIsNone(meymayakkam8("சர்ச்சை"))
        self.assertIsNone(meymayakkam8("தடம்"))

    def test_meymayakkam9(self):
        # Correct Words
        self.assertTrue(meymayakkam9("கப்பல் "))
        self.assertTrue(meymayakkam9("குப்பை"))
        self.assertTrue(meymayakkam9("சிப்பம்"))
        self.assertTrue(meymayakkam9("கப்பம்"))
        # Detecting Incorrect words
        self.assertFalse(meymayakkam9("யாப்தய"))
        self.assertFalse(meymayakkam9("ஞப்ழச"))
        # Detecting Not Applicable words
        self.assertIsNone(meymayakkam9("பத்து"))
        self.assertIsNone(meymayakkam9("வந்தான்"))
        self.assertIsNone(meymayakkam9("குரங்கு"))

    def test_meymayakkam10(self):
        # Correct Words
        self.assertTrue(meymayakkam10("அம்மை"))
        self.assertTrue(meymayakkam10("கம்பன் "))
        self.assertTrue(meymayakkam10("புலம்யாது"))
        self.assertTrue(meymayakkam10("வலம்வரும்"))
        # Detecting Incorrect words
        self.assertFalse(meymayakkam10("பழம்ரச"))
        self.assertFalse(meymayakkam10("கம்ளத"))
        # Detecting Not Applicable words
        self.assertIsNone(meymayakkam10("செந்நீர்"))
        self.assertIsNone(meymayakkam10("மஞ்சள்"))
        self.assertIsNone(meymayakkam10("அஞ்சறைப்பெட்டி"))

    def test_meymayakkam11(self):
        # Correct Words
        self.assertTrue(meymayakkam11("பொய்கை"))
        self.assertTrue(meymayakkam11("வேய்சிறிது"))
        self.assertTrue(meymayakkam11("வேய்ஞான்ற "))
        self.assertTrue(meymayakkam11("நொய்து"))
        self.assertTrue(meymayakkam11("மெய்நீண்டது"))
        self.assertTrue(meymayakkam11("மெய்பெரிது"))
        self.assertTrue(meymayakkam11("பேய்மனம்"))
        self.assertTrue(meymayakkam11("பேய்வலிது"))
        self.assertTrue(meymayakkam11("செய்யான் "))
#        self.assertTrue(meymayakkam11("வேய்ங்குழல்")) Having doubts here need to check with professor
        # Detecting Incorrect words
        self.assertFalse(meymayakkam11("காய்டமம்"))
        self.assertFalse(meymayakkam11("வய்றம்"))
        # Detecting Not Applicable words
        self.assertIsNone(meymayakkam11("அம்மை"))
        self.assertIsNone(meymayakkam11("நண்பகல் "))
        self.assertIsNone(meymayakkam11("நுட்பம்"))

    def test_meymayakkam12(self):
        # Correct Words
        self.assertTrue(meymayakkam12("வேர்கடிது"))
        self.assertTrue(meymayakkam12("வேர்ஙனம்"))
        self.assertTrue(meymayakkam12("வேர்சிறிது"))
        self.assertTrue(meymayakkam12("வேர்ஞான்றது"))
        self.assertTrue(meymayakkam12("வேர்நீண்டது"))
        self.assertTrue(meymayakkam12("வேர்பெரிது"))
        self.assertTrue(meymayakkam12("வேர்மாண்டது"))
        self.assertTrue(meymayakkam12("வேர்யாது"))
        self.assertTrue(meymayakkam12("வேர்வலிது"))
        # Detecting Incorrect words
        self.assertFalse(meymayakkam12("ராமராஜ்ஜியம்"))
        self.assertFalse(meymayakkam12("சாம்ராட்"))
        # Detecting Not Applicable words
        self.assertIsNone(meymayakkam12("சாம்பல்"))
        self.assertIsNone(meymayakkam12("வெண்மை"))
        self.assertIsNone(meymayakkam12("பச்சை"))

    def test_meymayakkam13(self):
        # Correct Words
        self.assertTrue(meymayakkam13("வீழ்கடிது"))
        self.assertTrue(meymayakkam13("வீழ்ஙனம்"))
        self.assertTrue(meymayakkam13("வீழ்சிறிது"))
        self.assertTrue(meymayakkam13("வீழ்ஞான்றது"))
        self.assertTrue(meymayakkam13("வீழ்தீது"))
        self.assertTrue(meymayakkam13("வீழ்நீண்டது"))
        self.assertTrue(meymayakkam13("வீழ்பெரிது"))
        self.assertTrue(meymayakkam13("வீழ்மாண்டது"))
        self.assertTrue(meymayakkam13("வீழ்யானை"))
        self.assertTrue(meymayakkam13("வாழ்வோர்"))

        # Detecting Incorrect words
        self.assertFalse(meymayakkam13("சூழ்ச்சி"))
        self.assertFalse(meymayakkam13("காழ்ப்புணர்ச்சி"))
        # Detecting Not Applicable words
        self.assertIsNone(meymayakkam13("பட்டை"))
        self.assertIsNone(meymayakkam13("வட்டம்"))
        self.assertIsNone(meymayakkam13("சதுரம்"))
        
    def test_meymayakkam14(self):
         # Correct Words
         self.assertTrue(meymayakkam14("செவ்வானம்"))
         # Detecting Not Applicable words
         self.assertIsNone(meymayakkam14("செங்காந்தள்"))

    def test_meymayakkam15(self):
        # Correct Words
        self.assertTrue(meymayakkam15("செல்க"))
        self.assertTrue(meymayakkam15("வல்சி"))
        self.assertTrue(meymayakkam15("செல்ப"))
        self.assertTrue(meymayakkam15("கொல்யானை"))
        self.assertTrue(meymayakkam15("கோல்வளை"))
        # Detecting Incorrect words
        self.assertFalse(meymayakkam15("செல்மாண்பு"))
        self.assertFalse(meymayakkam15("கொல்தவிடு"))
        # Detecting Not Applicable words
        self.assertIsNone(meymayakkam15("செவ்வானம்"))

    def test_meymayakkam16(self):
        # Correct Words
        self.assertTrue(meymayakkam16("கொள்க"))
        self.assertTrue(meymayakkam16("நீள்சினை"))
        self.assertTrue(meymayakkam16("கொள்ப"))
        self.assertTrue(meymayakkam16("வெள்ளை"))
        self.assertTrue(meymayakkam16("வெள்யாறு"))
        self.assertTrue(meymayakkam16("கள்வன்"))
        # Detecting Incorrect words
        self.assertFalse(meymayakkam16(""))
        self.assertFalse(meymayakkam16(""))
        # Detecting Not Applicable words
        self.assertIsNone(meymayakkam16(""))
        self.assertIsNone(meymayakkam16(""))
        self.assertIsNone(meymayakkam16(""))

    def test_meymayakkam17(self):
        # Correct Words
        self.assertTrue(meymayakkam17("கற்க"))
        self.assertTrue(meymayakkam17("முயற்சி"))
        self.assertTrue(meymayakkam17("கற்ப"))
        self.assertTrue(meymayakkam17("கற்றாழை"))
        # Detecting Incorrect words
        self.assertFalse(meymayakkam17("இதற்க்காக"))
        # Detecting Not Applicable words
        self.assertIsNone(meymayakkam17("கண்ணம்"))
        self.assertIsNone(meymayakkam17("கிண்ணம்"))
        self.assertIsNone(meymayakkam17("வண்ணம்"))

    def test_meymayakkam18(self):
        # Correct Words
        self.assertTrue(meymayakkam18("புன்கண்"))
        self.assertTrue(meymayakkam18("புன்செய்"))
        self.assertTrue(meymayakkam18("மென்ஞாண்"))
        self.assertTrue(meymayakkam18("அன்பு"))
        # Detecting Incorrect words
        self.assertFalse(meymayakkam18("என்தங்கை"))
        self.assertFalse(meymayakkam18("எம்புதல்வன்"))
        # Detecting Not Applicable words
        self.assertIsNone(meymayakkam18("சத்ரு"))
        self.assertIsNone(meymayakkam18("சுபாவம்"))
        self.assertIsNone(meymayakkam18("கண்மணி"))

#    def test_meymayakkam_(self):
#        # Correct Words
#        self.assertTrue(meymayakkam_(""))
#        self.assertTrue(meymayakkam_(""))
#        self.assertTrue(meymayakkam_(""))
#        self.assertTrue(meymayakkam_(""))
#        # Detecting Incorrect words
#        self.assertFalse(meymayakkam_(""))
#        self.assertFalse(meymayakkam_(""))
#        # Detecting Not Applicable words
#        self.assertIsNone(meymayakkam_(""))
#        self.assertIsNone(meymayakkam_(""))
#        self.assertIsNone(meymayakkam_(""))




from tamilstring import String
from tamilstring.utf8 import make_letter
from . import posTag


def specific_pointing_letters(data):
    """
    it removes `மை` at ending of any word and gives the result of the words
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    changes = False
    for index, (first_word, second_word) in enumerate(
        zip(data.copy[:-1], data.copy[1:])
    ):
        if first_word.string == "அ":
            if second_word.singleton(0).consonent in ["ந்", "ஞ்", "ம்"]:
                data.copy[index] = String(
                    first_word + second_word.singleton(0).consonent
                )
                data.rule = "remove_ending_consonent_mai"
            if second_word.singleton(0).consonent in ["ய்", "வ்"]:
                data.copy[index] = String(first_word + "வ்")
                data.rule = "remove_ending_consonent_mai"
            if second_word.singleton(0).kind == "VOL":
                data.copy[index] = String(first_word + "வ்வ்")
                data.rule = "remove_ending_consonent_mai"

    return (changes, data.status())


def cardinal_specific_degit(data):
    """
    it removes `மை` at ending of any word and gives the result of the words
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    changes = False
    for index, (first_word, second_word) in enumerate(
        zip(data.copy[:-1], data.copy[1:])
    ):
        if second_word.string == "குறை":
            if first_word.string in ["கா", "பனை"]:
                data.copy.insert(1, String("இன்"))
                data.rule = "remove_ending_consonent_mai"
            elif data.src[0] in ["கலம்"]:
                data.copy.insert(1, String("த்து"))
                data.rule = "remove_ending_consonent_mai"
            elif first_word.string in posTag.cardinal_degit:
                data.copy.insert(1, String("இன்"))
                data.rule = "remove_ending_consonent_mai"
            elif first_word.string in posTag.numbers_in_words:
                data.copy.insert(1, String("இன்"))
                data.rule = "remove_ending_consonent_mai"
    return (changes, data.status())


def specific_pointing_num(data):
    changes = False
    for index, (first_word, second_word) in enumerate(
        zip(data.copy[:-1], data.copy[1:])
    ):
        if data.src[0] == "நும்":
            if second_word.string == "அது":
                changes = True
                first_word.string = "நும்"
                data.rule = "specific_pointing_num"
            else:
                changes = True
                first_word.string = "நும"
                data.rule = "specific_pointing_num"
    return (changes, data.status())


def specific_pointing_list(data):
    changes = False
    for index, (first_word, second_word) in enumerate(
        zip(data.copy[:-1], data.copy[1:])
    ):
        print(make_letter(first_word.singleton(0).consonent, "அ"))
        if data.src[0] in ["தாம்", "நாம்"]:
            changes = True
            first_word = String(
                make_letter(first_word.singleton(0).consonent, "அ") + "ம"
            )
            data.rule = "specific_pointing_list"
        elif data.src[0] == "யான்":
            changes = True
            first_word = String("எம")
            data.rule = "specific_pointing_list"
    return (changes, data.status())


def apply(data):
    """
    it apply all the common rules and give the result of the words
    Args:
        data (WordsGenerator): list of words that you want to apply this grammar rule
    Return:
        list: [bool,dict]
    """
    status = []

    grammar_methods = [
        specific_pointing_letters,
        cardinal_specific_degit,
        specific_pointing_num,
        specific_pointing_list,
    ]

    for grammar in grammar_methods:
        return_responce = grammar(data)
        if return_responce[0] == True:
            status.append(return_responce[1])

    return status



import unittest
from tamilrulepy.euphonic import get
from tamilrulepy.thogaimarabu.thogaimarabu import thogai_6, thogai_7, thogai_8
from tamilrulepy.thogaimarabu.thogaimarabu import (
    thogai_1,
    thogai_2,
    thogai_3,
    thogai_4,
    thogai_5,
)


class Test_thogaimarabu(unittest.TestCase):

    def test_thogai_rule_1(self):
        result = get(["தட", "தோள்"])
        self.assertIn(["தடந்தோள்"], result)
        result = get(["கருமை", "சூழ்"])
        self.assertIn(["கருஞ்சூழ்"], result)

    def test_thogai_rule_2(self):
        result = get(["முருகன்", "மலை"])
        self.assertIn(["முருகன்மலை"], result)
        # result = get( ["அறம்","யாது"])
        # self.assertIn(['கருஞ்சூழ்'],result)
        result = get(["தமிழ்", "வாளர்கிறது"])
        self.assertIn(["தமிழ்வாளர்கிறது"], result)

    def test_thogai_1(self):
        output = thogai_1("தட", "தோள்")
        self.assertEqual("தடந்தோள்", output)
        # output2 = thogai_1("கருமை", "சூழ்")
        # self.assertEqual("கருஞ்சூழ்", output2) TODO: Need to check with professor. He removed mai at first word last
        output3 = thogai_1("உயிர்மெய்", "எழுத்துகள்")
        self.assertIsNone(output3)
        output4 = thogai_1("கருப்பு", "எருமை")
        self.assertIsNone(output4)

    def test_thogai_2(self):
        output = thogai_2("முருகன்", "மலை")
        self.assertEqual("முருகன் மலை", output)
        output2 = thogai_2("அறம்", "யாது")
        self.assertEqual("அறம் யாது", output2)
        output3 = thogai_2("அறம்", "சிலை")
        self.assertIsNone(output3)
        with self.assertRaises(ValueError):
            thogai_2("அறம்ஃ", "யாது")

    def test_thogai_3(self):
        output = thogai_3("மெய்", "ஞானம்")
        self.assertEqual("மெய்ஞ்ஞானம்", output)
        output2 = thogai_3("கை", "நொடி")
        self.assertEqual("கைந்நொடி", output2)
        with self.assertRaises(ValueError):
            thogai_3("அறம்ஃ", "யாது")
        output3 = thogai_3("கை", "சிலை")
        self.assertIsNone(output3)

    def test_thogai_4(self):
        output = thogai_4("மண்", "யாத்த")
        self.assertEqual("மண்ஞாத்த", output)
        output2 = thogai_4("பொன்", "யாத்த")
        self.assertEqual("பொன்ஞாத்த", output2)
        output3 = thogai_4("கை", "நொடி")
        self.assertIsNone(output3)
        output4 = thogai_4("மண்", "ஆத்த")
        self.assertIsNone(output4)

    def test_thogai_5(self):
        output = thogai_5("மண்", "வலிது")
        self.assertEqual("மண்வலிது", output)
        output2 = thogai_5("பொன்", "கடிது")
        self.assertEqual("பொன்கடிது", output2)
        with self.assertRaises(ValueError):
            thogai_5("அறம்", "ஃயாது")
        output3 = thogai_5("கை", "நொடி")
        self.assertIsNone(output3)

    # def test_thogai_rule_3(self):
    # result = get(["கை","நொடி"])
    # self.assertIn(['தடந்தோள்'], result)
    # result = get(["மேய்","ஞானம்"])
    # self.assertIn(['தடந்தோள்'], result)

    def test_thogai_rule_4(self):
        result = get(["மண்", "யாத்த"])
        self.assertIn(["மண்ஞாத்த"], result)
        result = get(["பொன்", "யாத்த"])
        self.assertIn(["பொன்ஞாத்த"], result)

    def test_thogai_rule_5(self):
        result = get(["மண்", "கடிது"])
        # self.assertIn(["மண்கடிது"], result)
        result = get(["மண்", "வலிது"])
        self.assertIn(["மண்வலிது"], result)
        result = get(["பொன்", "கடிது"])
        # self.assertIn(["பொன்கடிது"], result)
        result = get(["பொன்", "வலிது"])
        self.assertIn(["பொன்வலிது"], result)

    # unittest for functions written by hariharan U -- starts 
    def test_thogai_rule_6(self):

        word_list = [
            ("பொன்","குடம்","பொற்குடம்"),
            ("மண்","துனை","மட்துனை"),
            ("மண்","ஞாலம்","மண்ஞாலம்"),
            ("பொன்","விலை","பொன்விலை"),
            ("பொன்","சிலை","பொற்சிலை"),
            ("மண்","பாண்டம்","மட்பாண்டம்"),
            ("வின்","அரசன்","வின்அரசன்"),
            ("தன்","பெறுமை","தற்பெறுமை")
            ]
        
        for first_word,second_word,result in word_list:
            self.assertIn(thogai_6(first_word,second_word),result)
        
    def test_thogai_rule_7(self):
        word_list = [
            ("பொன்","தாலி","பொற்றாலி"),
            ("கல்","தரை","கற்றரை"),
            ("தன்","நிலை","தன்னிலை"),
            ("கல்","தவளை","கற்றவளை"),
            ("பொன்","திரை","பொற்றிரை"),
            ("தன்","தரப்பு","தற்றரப்பு"),
            ("சிலம்பன்","தை","சிலம்பற்றை")
            ]
        for first_word,second_word,result in word_list:
                self.assertIn(thogai_7(first_word,second_word),result)
        
    def test_thogai_rule_8(self):
        #boobalan's testcases for euphonic 
        """ result =  get(["மண்","தீது"])  #bug
        self.assertIn(["மட்டீது"], result) 
        result =  get(["மண்","நன்று"]) 
        self.assertIn(["மண்ணன்று"], result) 
        result =  get(["மண்","நினைவு"])
        self.assertIn(["மண்ணினைவு"], result)

        result =  get(["கள்","தீது"]) 
        self.assertIn(["கட்டது"], result) 
        result =  get(["கள்","நன்று"]) 
        self.assertIn(["கண்ணின்று"], result) 
        result =  get(["கள்","நினைவு"])
        self.assertIn(["கண்ணினைவு"], result)

        result =  get(["உள்","தீது"]) 
        self.assertIn(["உட்டீது"], result) 
        result =  get(["உள்","நன்று"]) 
        self.assertIn(["உண்ணன்று"], result) 
        result =  get(["உள்","நினைவு"])
        self.assertIn(["உண்ணினைவு"], result) """
        # thogaimarabu individual function testcases
        word_list = [
            ("மண்","தீது","மட்டீது"),
            ("மண்","நன்று","மண்ணன்று"),
            ("கள்","தீது","கட்டீது"),
            ("உள்","நினைவு","உண்ணினைவு"),
            ("ஊண்","தின்று","ஊட்டின்று")
        ]
        for first_word,second_word,result in word_list: 
            self.assertIn(thogai_8(first_word,second_word),result)


"""
    def test_thogai_rule_17(self):
        result = get("அதோனி","கொண்டான்")
        self.assertIn("அதோனிக்கொண்டான்")
        result = get("இதோனி","கொண்டான்")
        self.assertIn("இதோனிக்கொண்டான்")
        result = get("உதோனி","கொண்டான்")
        self.assertIn("உதோனிக்கொண்டான்")
        result = get("எதோனி","கொண்டான்")
        self.assertIn("எதோனிக்கொண்டான்")
        result = get("ஆண்டை","கொண்டான்")
        self.assertIn("ஆண்டைக்கொண்டான்")
        result = get("யாண்டை","கொண்டான்")
        self.assertIn("யாண்டைக்கொண்டான்")
    
    def test_thogai_rule_19(self):
        result =  get(["தாம்","கு"]) 
        self.assertIn(["தமக்கு"], result) 
        result =  get(["நாம்","கு"]) 
        self.assertIn(["நமக்கு"], result) 
        result =  get(["யான்","கு"]) 
        self.assertIn(["எனக்கு"], result) 

        result =  get(["தாம்","அது"]) 
        self.assertIn(["தமது"], result) 
        result =  get(["நாம்","அது"]) 
        self.assertIn(["நமது"], result) 
        result =  get(["யான்","அது"]) 
        self.assertIn(["எனது"], result) 
   
    def test_thogai_rule_20(self):
        result =  get(["நும்","கு"]) 
        self.assertIn(["நுமக்கு"], result) 
        result =  get(["நும்","அது"]) 
        self.assertIn(["நுமது"], result) 
    
    def test_thogai_rule_21(self):
        result =  get(["உரிஞு","யாது"]) 
        self.assertIn(["உரிஞ்யாது"], result) 
        result =  get(["பொருநு","யாது"]) 
        self.assertIn(["பொருந்யாது"], result) 
        result =  get(["உரிஞு","அழகா"]) 
        self.assertIn(["உரிஞ்அழகா"], result) 
        result =  get(["பொருநு","அழகா"]) 
        self.assertIn(["பொருந்அழகா"], result) 

    def test_thogai_rule_21(self):
        pass

    def test_thogai_rule_23(self):
        result =  get(["ஒன்று","அரை"]) 
        self.assertIn(["ஒன்றரை"], result) 
        result =  get(["ஏழு","அரை"]) 
        self.assertIn(["ஏழரை"], result) 


    def test_thogai_rule_24(self):
        result =  get(["உரி","குறை"]) 
        self.assertIn(["உரிக்குறை"], result) 
        result =  get(["காணி","குறை"]) 
        self.assertIn(["காணிக்குறை"], result) 


    def test_thogai_rule_25(self):
        result =  get(["உழக்கு","குறை"]) 
        self.assertIn(["உழக்கின்குறை"], result) 
        result =  get(["ஒன்று","குறை"]) 
        self.assertIn(["ஒன்றின்குறை"], result) 

    def test_thogai_rule_26(self):
        result =  get(["கலம்","குறை"]) 
        self.assertIn(["கலத்துக்குறை"], result) 
      
    def test_thogai_rule_27(self):
        result =  get(["பனை","குறை"]) 
        #self.assertIn(["பனையின்குறை"], result)  # பனைவின்குறை
        result =  get(["கா","குறை"]) 
        self.assertIn(["காவின்குறை"], result) 

"""
        



from typing import List, Final

# For Tamil string literals
TamilString = str

# Case suffixes
Case: Final[List[TamilString]] = ["கு", "கண்", "அது"]

# Tamil particles
Particals: Final[List[TamilString]] = ["இன்", "வற்று", "அத்து", "அம்", "ஆன்", "அக்கு", "இக்கு"]


# numerical words representation

numbers_in_words = [
    "ஒன்று",
    "இரண்டு",
    "மூன்று",
    "நான்கு",
    "ஐந்து",
    "ஆறு",
    "ஏழு",
    "எட்டு",
    "ஒன்பது",
    "பத்து",
]

# Cardinal numbers
cardinal_degit: Final[List[TamilString]] = [
    "உழக்கு",
    "பானை",
    "கா",
    "ஆலாக்கு",
    "அகழ்",
    "அங்குலம்",
]


# Wh-adverbs (interrogative/relative adverbs)
wh_adverb: Final[List[TamilString]] = ["அவை", "இவை", "எவை", "அது", "இது", "எது"]

wh_adverb_e_i = [
    "அதோனி",
    "இதோனி",
    "உதோனி",
    "எதோனி",
    "ஆண்டை",
    "யாண்டை",
]

uyartiṇai = "யாவர்"


list_of_words = ["தாம்", "நாம்", "யான்"]



from tamilstring import get_letters
import sys
from .stackwords import WordsGenerator, Maintainer


try:
    from IPython.display import display, Markdown

    IPYTHON_AVAILABLE = True
except ImportError:
    IPYTHON_AVAILABLE = False


def get(words_list, rules=False, IPY=True, lang="en"):
    """
    it is the common method to apply and get all the possible form of words that are derived by tamil grammar rules.
    Args:
        data (list): list of words that you want to apply grammar rules
        rules (bool): defalut is False, if you enable it as True, it will shows you what are the grammar rules that are applied by this library
        IPY (bool): default is True, it only works on Ipython. it helps to give output in ipython in a orginized way to visualize.
    Return:
        list:  by default it returns a list of list contains tamil words that are derived by tamil grammar rules
    """

    data = Maintainer(words_list, lang)

    finalizer = WordsGenerator()

    finalizer.config_grammer_rules(data)

    if rules:
        if "ipykernel" in sys.modules and IPY:
            return display(Markdown(convert_markdown_table(finalizer.get_words())))
        else:
            return finalizer.get_words()

    else:
        return [sublist[-1]["words"] for sublist in finalizer.get_words() if sublist]


def convert_markdown_table(data):
    tables = []

    for i, sublist in enumerate(data, start=1):
        headers = ["Words", "Rule"]
        md_table = f"### Table {i}\n\n"
        md_table += "| " + " | ".join(headers) + " |\n"
        md_table += "|---" * len(headers) + "|\n"

        for entry in sublist:
            words_str = " + ".join(entry["words"])
            rule_str = entry["rule"] if entry["rule"] is not None else "None"
            md_table += f"| {words_str} | {rule_str} |\n"

        tables.append(md_table)

    return "\n\n".join(tables)



# Importing the unittest package
import unittest
# Importing the module to be tested
from tamilrulepy.mozhimarabu import  word_starting


# சோதனை செய்ய பயன்படுத்தப்பட்ட சொற்ககளுக்கான மூல தொடுப்பு 
# https://www.tamilvu.org/courses/degree/c021/c0211/html/c0211502.htm

# Test case definition

class TestWordStarting(unittest.TestCase):

    def test_uyirezhuthu_check(self):
        data_list = ['அன்பு','அறிவு','அகந்தை','வலை']
        expected = [True,True,True,None]
        res = map(word_starting.uyirezhuthu_check,data_list)
        actual = list(res)
        self.assertEqual(expected,actual)

    def test_uyirmei_ka_check(self):
        data_list = ['அறிவு','கல்வி','காண்','கிளி','கீறு','குறிப்பு','கூகை','கெஞ்சு','கேள்வி','கை','கொப்பறை','கோவில்','கௌரி']
        expected = [None,True,True,True,True,True,True,True,True,True,True,True,True]
        res = map(word_starting.uyirmei_ka_check,data_list)
        actual =list(res)
        self.assertEqual(expected,actual)

    def test_uyirmei_sa_check(self):
        data_list = [
            "அறிவு",
            "ச",
            "சாப்பிடு",
            "சி",
            "சீ",
            "சு",
            "சூ",
            "செ",
            "சே",
            "சொ",
            "சோ"]
        expected = [None,True,True,True,True,True,True,True,True,True,True]
        res = map(word_starting.uyirmei_sa_check,data_list)
        actual =list(res)
        self.assertEqual(expected,actual)

    def test_uyirmei_nga_check(self):
        data_list = ["ஞமலி","ஞாலம்", "ஞெகிழி", "ஞொள்குதல்"]
        expected = [None,True,True,True]
        res = map(word_starting.uyirmei_nga_check,data_list)
        actual =list(res)
        self.assertEqual(expected,actual)

    def test_uyirmei_ta_check(self):
        data_list = ["ஸிர்","கண்ணன்","தரை","தாமரை","திசை","தீர்ப்பு","துடிப்பு","தூண்","தென்னைமரம்","தேன்","தொழில்","தோட்டம்","தௌவை"]
        expected = [None,None,True,True,True,True,True,True,True,True,True,True,True]
        res = map(word_starting.uyirmei_ta_check,data_list)
        actual =list(res)
        self.assertEqual(expected,actual)

    def test_uyirmei_na_check(self):
        data_list = ["ஜூம்பா","கதிரவன்","நன்றி","நாடு","நிறம்","நீர்","நுங்கு","நூல்","நெல்","நேற்று","நையாண்டி","நொடி","நோக்கம்","நௌவி"]
        expected = [None,None,True,True,True,True,True,True,True,True,True,True,True,True]
        res = map(word_starting.uyirmei_na_check,data_list)
        actual =list(res)
        self.assertEqual(expected,actual)

    def test_uyirmei_pa_check(self):
        data_list = ["குறிச்சி","ஜோ","பல்","பால்","பிடி","பீலி","புகழ்","பூங்கா","பெட்டி","பேச்சு","பை","பொன்","போட்டி","பௌத்தர்"]
        expected = [None,None,True,True,True,True,True,True,True,True,True,True,True,True]
        res = map(word_starting.uyirmei_pa_check,data_list)
        actual =list(res)
        self.assertEqual(expected,actual)

    def test_uyirmei_ma_check(self):
        data_list = ["குறிச்சி","ஜோ","மண்","மான்","மின்னல்","மீன்","முரசு","மூங்கில்","மெய்","மேடு","மை","மொழி","மோதிரம்","மௌனம்"]
        expected = [None,None,True,True,True,True,True,True,True,True,True,True,True,True]
        res = map(word_starting.uyirmei_ma_check,data_list)
        actual =list(res)
        self.assertEqual(expected,actual)

    def test_uyirmei_ya_check(self):
        data_list = ["குறிச்சி","ஜோ","யானை","யாழ்"]
        expected = [None,None,True,True]
        res = map(word_starting.uyirmei_ya_check,data_list)
        actual =list(res)
        self.assertEqual(expected,actual)

    def test_uyirmei_va_check(self):
        data_list =  ["குறிச்சி","ஜோ","வணக்கம்", "வால்", "வில்", "வீடு", "வெற்றி", "வேல்", "வைகை", "வௌவால்"]
        expected = [None,None,True,True,True,True,True,True,True,True]
        res = map(word_starting.uyirmei_va_check,data_list)
        actual =list(res)
        self.assertEqual(expected,actual)
    



